@@ -3,7 +3,7 @@
  */
 
 /*
- * Copyright (C) 2006-2011 Oracle Corporation
+ * Copyright (C) 2006-2012 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -52,6 +52,8 @@ typedef enum RTLOGGROUP
 {
     /** Default logging group. */
     RTLOGGROUP_DEFAULT,
+    RTLOGGROUP_DBG,
+    RTLOGGROUP_DBG_DWARF,
     RTLOGGROUP_DIR,
     RTLOGGROUP_FILE,
     RTLOGGROUP_FS,
@@ -81,6 +83,8 @@ typedef enum RTLOGGROUP
  */
 #define RT_LOGGROUP_NAMES \
     "DEFAULT",      \
+    "RT_DBG",       \
+    "RT_DBG_DWARF", \
     "RT_DIR",       \
     "RT_FILE",      \
     "RT_FS",        \
@@ -91,8 +95,6 @@ typedef enum RTLOGGROUP
     "RT_THREAD",    \
     "RT_TIME",      \
     "RT_TIMER",     \
-    "RT_11", \
-    "RT_12", \
     "RT_13", \
     "RT_14", \
     "RT_15", \
@@ -703,6 +705,22 @@ RTDECL(void) RTLogPrintfEx(void *pvInstance, unsigned fFlags, unsigned iGroup, c
     do { Log((LOG_FN_FMT ": ", __PRETTY_FUNCTION__)); Log(a); } while (0)
 #endif
 
+/** @def Log4Func
+ * Level 4 logging inside C/C++ functions.
+ *
+ * Prepends the given log message with the function name followed by a
+ * semicolon and space.
+ *
+ * @param   a   Log message in format <tt>("string\n" [, args])</tt>.
+ */
+#ifdef LOG_USE_C99
+# define Log4Func(a) \
+    _LogIt(LOG_INSTANCE, RTLOGGRPFLAGS_LEVEL_4, LOG_GROUP, LOG_FN_FMT ": %M", __PRETTY_FUNCTION__, _LogRemoveParentheseis a )
+#else
+# define Log4Func(a) \
+    do { Log((LOG_FN_FMT ": ", __PRETTY_FUNCTION__)); Log(a); } while (0)
+#endif
+
 /** @def LogThisFunc
  * The same as LogFunc but for class functions (methods): the resulting log
  * line is additionally prepended with a hex value of |this| pointer.
@@ -1642,9 +1660,9 @@ RTDECL(int) RTLogGetGroupSettings(PRTLOGGER pLogger, char *pszBuf, size_t cchBuf
  * @returns iprt status code.
  *          Failures can safely be ignored.
  * @param   pLogger     Logger instance (NULL for default logger).
- * @param   pszVar      Value to parse.
+ * @param   pszValue    Value to parse.
  */
-RTDECL(int) RTLogGroupSettings(PRTLOGGER pLogger, const char *pszVar);
+RTDECL(int) RTLogGroupSettings(PRTLOGGER pLogger, const char *pszValue);
 #endif /* !IN_RC */
 
 /**
@@ -1654,9 +1672,9 @@ RTDECL(int) RTLogGroupSettings(PRTLOGGER pLogger, const char *pszVar);
  * @returns iprt status code.
  *          Failures can safely be ignored.
  * @param   pLogger     Logger instance (NULL for default logger).
- * @param   pszVar      Value to parse.
+ * @param   pszValue    Value to parse.
  */
-RTDECL(int) RTLogFlags(PRTLOGGER pLogger, const char *pszVar);
+RTDECL(int) RTLogFlags(PRTLOGGER pLogger, const char *pszValue);
 
 /**
  * Changes the buffering setting of the specified logger.
@@ -1701,9 +1719,9 @@ RTDECL(int) RTLogGetFlags(PRTLOGGER pLogger, char *pszBuf, size_t cchBuf);
  *
  * @returns VINF_SUCCESS or VERR_BUFFER_OVERFLOW.
  * @param   pLogger             Logger instance (NULL for default logger).
- * @param   pszVar              The value to parse.
+ * @param   pszValue            The value to parse.
  */
-RTDECL(int) RTLogDestinations(PRTLOGGER pLogger, char const *pszVar);
+RTDECL(int) RTLogDestinations(PRTLOGGER pLogger, char const *pszValue);
 
 /**
  * Get the current log destinations as a string.
