@@ -45,9 +45,13 @@ RT_C_DECLS_BEGIN
  */
 typedef union RTSHA1CONTEXT
 {
-    uint8_t abPadding[ARCH_BITS == 32 ? 96 : 128];
+    uint64_t                u64BetterAlignment;
+    uint8_t                 abPadding[8 + (5 + 80) * 4 + 4];
 #ifdef RT_SHA1_PRIVATE_CONTEXT
-    SHA_CTX Private;
+    SHA_CTX                 Private;
+#endif
+#ifdef RT_SHA1_PRIVATE_ALT_CONTEXT
+    RTSHA1ALTPRIVATECTX     AltPrivate;
 #endif
 } RTSHA1CONTEXT;
 /** Pointer to an SHA-1 context. */
@@ -138,6 +142,7 @@ RTR3DECL(int) RTSha1Digest(void* pvBuf, size_t cbBuf, char **ppszDigest, PFNRTPR
 RTR3DECL(int) RTSha1DigestFromFile(const char *pszFile, char **ppszDigest, PFNRTPROGRESS pfnProgressCallback, void *pvUser);
 
 
+
 /** The size of a SHA-256 hash. */
 #define RTSHA256_HASH_SIZE      32
 /** The length of a SHA-256 digest string. The terminator is not included. */
@@ -148,9 +153,13 @@ RTR3DECL(int) RTSha1DigestFromFile(const char *pszFile, char **ppszDigest, PFNRT
  */
 typedef union RTSHA256CONTEXT
 {
-    uint8_t abPadding[ARCH_BITS == 32 ? 112 : 160];
+    uint64_t                u64BetterAlignment;
+    uint8_t                 abPadding[8 + (8 + 80) * 4];
 #ifdef RT_SHA256_PRIVATE_CONTEXT
-    SHA256_CTX Private;
+    SHA256_CTX              Private;
+#endif
+#ifdef RT_SHA256_PRIVATE_ALT_CONTEXT
+    RTSHA256ALTPRIVATECTX   AltPrivate;
 #endif
 } RTSHA256CONTEXT;
 /** Pointer to an SHA-256 context. */
@@ -242,6 +251,104 @@ RTR3DECL(int) RTSha256Digest(void* pvBuf, size_t cbBuf, char **ppszDigest, PFNRT
  */
 RTR3DECL(int) RTSha256DigestFromFile(const char *pszFile, char **ppszDigest, PFNRTPROGRESS pfnProgressCallback, void *pvUser);
 
+
+
+/** The size of a SHA-224 hash. */
+#define RTSHA224_HASH_SIZE      28
+/** The length of a SHA-224 digest string. The terminator is not included. */
+#define RTSHA224_DIGEST_LEN     56
+
+/** SHA-224 context (same as for SHA-256). */
+typedef RTSHA256CONTEXT RTSHA224CONTEXT;
+/** Pointer to an SHA-224 context. */
+typedef RTSHA256CONTEXT *PRTSHA224CONTEXT;
+
+/**
+ * Compute the SHA-224 hash of the data.
+ *
+ * @param   pvBuf       Pointer to the data.
+ * @param   cbBuf       The amount of data (in bytes).
+ * @param   pabDigest   Where to store the hash. (What is passed is a pointer to
+ *                      the caller's buffer.)
+ */
+RTDECL(void) RTSha224(const void *pvBuf, size_t cbBuf, uint8_t pabDigest[RTSHA224_HASH_SIZE]);
+
+/**
+ * Initializes the SHA-224 context.
+ *
+ * @param   pCtx        Pointer to the SHA-224 context.
+ */
+RTDECL(void) RTSha224Init(PRTSHA224CONTEXT pCtx);
+
+/**
+ * Feed data into the SHA-224 computation.
+ *
+ * @param   pCtx        Pointer to the SHA-224 context.
+ * @param   pvBuf       Pointer to the data.
+ * @param   cbBuf       The length of the data (in bytes).
+ */
+RTDECL(void) RTSha224Update(PRTSHA224CONTEXT pCtx, const void *pvBuf, size_t cbBuf);
+
+/**
+ * Compute the SHA-224 hash of the data.
+ *
+ * @param   pCtx        Pointer to the SHA-224 context.
+ * @param   pabDigest   Where to store the hash. (What is passed is a pointer to
+ *                      the caller's buffer.)
+ */
+RTDECL(void) RTSha224Final(PRTSHA224CONTEXT pCtx, uint8_t pabDigest[RTSHA224_HASH_SIZE]);
+
+/**
+ * Converts a SHA-224 hash to a digest string.
+ *
+ * @returns IPRT status code.
+ *
+ * @param   pabDigest   The binary digest returned by RTSha224Final or RTSha224.
+ * @param   pszDigest   Where to return the stringified digest.
+ * @param   cchDigest   The size of the output buffer. Should be at least
+ *                      RTSHA224_DIGEST_LEN + 1 bytes.
+ */
+RTDECL(int) RTSha224ToString(uint8_t const pabDigest[RTSHA224_HASH_SIZE], char *pszDigest, size_t cchDigest);
+
+/**
+ * Converts a SHA-224 hash to a digest string.
+ *
+ * @returns IPRT status code.
+ *
+ * @param   pszDigest   The stringified digest. Leading and trailing spaces are
+ *                      ignored.
+ * @param   pabDigest   Where to store the hash. (What is passed is a pointer to
+ *                      the caller's buffer.)
+ */
+RTDECL(int) RTSha224FromString(char const *pszDigest, uint8_t pabDigest[RTSHA224_HASH_SIZE]);
+
+/**
+ * Creates a SHA224 digest for the given memory buffer.
+ *
+ * @returns iprt status code.
+ *
+ * @param   pvBuf                 Memory buffer to create a SHA224 digest for.
+ * @param   cbBuf                 The amount of data (in bytes).
+ * @param   ppszDigest            On success the SHA224 digest.
+ * @param   pfnProgressCallback   optional callback for the progress indication
+ * @param   pvUser                user defined pointer for the callback
+ */
+RTR3DECL(int) RTSha224Digest(void* pvBuf, size_t cbBuf, char **ppszDigest, PFNRTPROGRESS pfnProgressCallback, void *pvUser);
+
+/**
+ * Creates a SHA224 digest for the given file.
+ *
+ * @returns iprt status code.
+ *
+ * @param   pszFile               Filename to create a SHA224 digest for.
+ * @param   ppszDigest            On success the SHA224 digest.
+ * @param   pfnProgressCallback   optional callback for the progress indication
+ * @param   pvUser                user defined pointer for the callback
+ */
+RTR3DECL(int) RTSha224DigestFromFile(const char *pszFile, char **ppszDigest, PFNRTPROGRESS pfnProgressCallback, void *pvUser);
+
+
+
 /** The size of a SHA-512 hash. */
 #define RTSHA512_HASH_SIZE      64
 /** The length of a SHA-512 digest string. The terminator is not included. */
@@ -252,9 +359,13 @@ RTR3DECL(int) RTSha256DigestFromFile(const char *pszFile, char **ppszDigest, PFN
  */
 typedef union RTSHA512CONTEXT
 {
-    uint8_t abPadding[ARCH_BITS == 32 ? 216 : 256];
+    uint64_t                u64BetterAlignment;
+    uint8_t                 abPadding[16 + (80 + 8) * 8];
 #ifdef RT_SHA512_PRIVATE_CONTEXT
-    SHA512_CTX Private;
+    SHA512_CTX              Private;
+#endif
+#ifdef RT_SHA512_PRIVATE_ALT_CONTEXT
+    RTSHA512ALTPRIVATECTX   AltPrivate;
 #endif
 } RTSHA512CONTEXT;
 /** Pointer to an SHA-512 context. */
@@ -319,9 +430,44 @@ RTDECL(int) RTSha512ToString(uint8_t const pabDigest[RTSHA512_HASH_SIZE], char *
  */
 RTDECL(int) RTSha512FromString(char const *pszDigest, uint8_t pabDigest[RTSHA512_HASH_SIZE]);
 
+
+/** Macro for declaring the interface for a SHA-512 variation.
+ * @internal */
+#define RTSHA512_DECLARE_VARIANT(a_Name, a_UName) \
+    typedef RTSHA512CONTEXT RT_CONCAT3(RTSHA,a_UName,CONTEXT); \
+    typedef RTSHA512CONTEXT *RT_CONCAT3(PRTSHA,a_UName,CONTEXT); \
+    RTDECL(void) RT_CONCAT(RTSha,a_Name)(const void *pvBuf, size_t cbBuf, uint8_t pabDigest[RT_CONCAT3(RTSHA,a_UName,_HASH_SIZE)]); \
+    RTDECL(void) RT_CONCAT3(RTSha,a_Name,Init)(RT_CONCAT3(PRTSHA,a_UName,CONTEXT) pCtx); \
+    RTDECL(void) RT_CONCAT3(RTSha,a_Name,Update)(RT_CONCAT3(PRTSHA,a_UName,CONTEXT) pCtx, const void *pvBuf, size_t cbBuf); \
+    RTDECL(void) RT_CONCAT3(RTSha,a_Name,Final)(RT_CONCAT3(PRTSHA,a_UName,CONTEXT) pCtx, uint8_t pabDigest[RT_CONCAT3(RTSHA,a_UName,_HASH_SIZE)]); \
+    RTDECL(int)  RT_CONCAT3(RTSha,a_Name,ToString)(uint8_t const pabDigest[RT_CONCAT3(RTSHA,a_UName,_HASH_SIZE)], char *pszDigest, size_t cchDigest); \
+    RTDECL(int)  RT_CONCAT3(RTSha,a_Name,FromString)(char const *pszDigest, uint8_t pabDigest[RT_CONCAT3(RTSHA,a_UName,_HASH_SIZE)])
+
+
+/** The size of a SHA-384 hash. */
+#define RTSHA384_HASH_SIZE      48
+/** The length of a SHA-384 digest string. The terminator is not included. */
+#define RTSHA384_DIGEST_LEN     96
+RTSHA512_DECLARE_VARIANT(384,384);
+
+/** The size of a SHA-512/224 hash. */
+#define RTSHA512T224_HASH_SIZE  28
+/** The length of a SHA-512/224 digest string. The terminator is not
+ *  included. */
+#define RTSHA512T224_DIGEST_LEN 56
+RTSHA512_DECLARE_VARIANT(512t224,512T224);
+
+/** The size of a SHA-512/256 hash. */
+#define RTSHA512T256_HASH_SIZE  32
+/** The length of a SHA-512/256 digest string. The terminator is not
+ *  included. */
+#define RTSHA512T256_DIGEST_LEN 64
+RTSHA512_DECLARE_VARIANT(512t256,512T256);
+
+
 /** @} */
 
 RT_C_DECLS_END
 
-#endif /* ___iprt_sha1_h */
+#endif
 
