@@ -97,6 +97,13 @@
 # endif
 #endif
 
+/* The first of these was introduced when drm was generalised to work with
+ * non-PCI buses, but was removed between 3.15 and 3.16.  The second is a
+ * random definition introduced in the mean-time. */
+#if defined(DRIVER_BUS_PCI) || defined(DRIVER_PRIME)
+# define DRM_NEW_BUS_INIT 1
+#endif
+
 static struct pci_device_id pciidlist[] = {
         vboxvideo_PCI_IDS
 };
@@ -105,11 +112,7 @@ MODULE_DEVICE_TABLE(pci, pciidlist);
 
 int vboxvideo_driver_load(struct drm_device * dev, unsigned long flags)
 {
-# if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 28)
-    return drm_vblank_init(dev, 1);
-#else
     return 0;
-#endif
 }
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 3, 0) || defined(DRM_FOPS_AS_POINTER)
 /* since linux-3.3.0-rc1 drm_driver::fops is pointer */
@@ -155,7 +158,7 @@ static struct drm_driver driver =
 #else /* LINUX_VERSION_CODE >= KERNEL_VERSION(3, 3, 0) || defined(DRM_FOPS_AS_POINTER) */
     .fops = &driver_fops,
 #endif
-#ifndef DRIVER_BUS_PCI
+#ifndef DRM_NEW_BUS_INIT
     .pci_driver =
     {
         .name = DRIVER_NAME,
@@ -170,7 +173,7 @@ static struct drm_driver driver =
     .patchlevel = DRIVER_PATCHLEVEL,
 };
 
-#ifdef DRIVER_BUS_PCI
+#ifdef DRM_NEW_BUS_INIT
 static struct pci_driver pci_driver =
 {
     .name = DRIVER_NAME,
@@ -180,7 +183,7 @@ static struct pci_driver pci_driver =
 
 static int __init vboxvideo_init(void)
 {
-#ifndef DRIVER_BUS_PCI
+#ifndef DRM_NEW_BUS_INIT
     return drm_init(&driver);
 #else
     return drm_pci_init(&driver, &pci_driver);
@@ -189,7 +192,7 @@ static int __init vboxvideo_init(void)
 
 static void __exit vboxvideo_exit(void)
 {
-#ifndef DRIVER_BUS_PCI
+#ifndef DRM_NEW_BUS_INIT
     drm_exit(&driver);
 #else
     drm_pci_exit(&driver, &pci_driver);
