@@ -243,6 +243,7 @@ AssertCompileMembersAtSameOffset(DEVSMC, u.abRegsRW[SMC_REG_STATUS_CODE], DEVSMC
 /** Pointer to the SMC state. */
 typedef DEVSMC *PDEVSMC;
 
+#ifndef VBOX_DEVICE_STRUCT_TESTCASE
 
 
 /**
@@ -1161,6 +1162,7 @@ static const struct
     /* [0x0d] = */ { smcRegFF_w,       smcRegFF_r },
     /* [0x0e] = */ { smcRegFF_w,       smcRegFF_r },
     /* [0x0f] = */ { smcRegFF_w,       smcRegFF_r },
+    /* [0x10] = */ { smcRegGen_w,      smcRegGen_r },
     /* [0x11] = */ { smcRegGen_w,      smcRegGen_r },
     /* [0x12] = */ { smcRegGen_w,      smcRegGen_r },
     /* [0x13] = */ { smcRegGen_w,      smcRegGen_r },
@@ -1427,3 +1429,4 @@ const PDMDEVREG g_DeviceSmc =
 };
 
 #endif /* IN_RING3 */
+#endif /* VBOX_DEVICE_STRUCT_TESTCASE */
