@@ -338,10 +338,6 @@ static DECLCALLBACK(void) vusbRhFreeUrb(PVUSBURB pUrb)
     vusbUrbAssert(pUrb);
     PVUSBROOTHUB pRh = (PVUSBROOTHUB)pUrb->VUsb.pvFreeCtx;
     Assert(pRh);
-#ifdef VBOX_STRICT
-    for (PVUSBURB pCur = pRh->pAsyncUrbHead; pCur; pCur = pCur->VUsb.pNext)
-        Assert(pUrb != pCur);
-#endif
 
     /*
      * Free the URB description (logging builds only).
@@ -356,13 +352,13 @@ static DECLCALLBACK(void) vusbRhFreeUrb(PVUSBURB pUrb)
      * Put it into the LIFO of free URBs.
      * (No ppPrev is needed here.)
      */
-    RTCritSectEnter(&pRh->CritSect);
+    RTCritSectEnter(&pRh->CritSectFreeUrbs);
     pUrb->enmState = VUSBURBSTATE_FREE;
     pUrb->VUsb.ppPrev = NULL;
     pUrb->VUsb.pNext = pRh->pFreeUrbs;
     pRh->pFreeUrbs = pUrb;
     Assert(pRh->pFreeUrbs->enmState == VUSBURBSTATE_FREE);
-    RTCritSectLeave(&pRh->CritSect);
+    RTCritSectLeave(&pRh->CritSectFreeUrbs);
 }
 
 
@@ -379,7 +375,7 @@ PVUSBURB vusbRhNewUrb(PVUSBROOTHUB pRh, uint8_t DstAddress, uint32_t cbData, uin
     /** @todo The allocations should be done by the device, at least as an option, since the devices
      * frequently wish to associate their own stuff with the in-flight URB or need special buffering
      * (isochronous on Darwin for instance). */
-    RTCritSectEnter(&pRh->CritSect);
+    RTCritSectEnter(&pRh->CritSectFreeUrbs);
     PVUSBURB pUrbPrev = NULL;
     PVUSBURB pUrb = pRh->pFreeUrbs;
     while (pUrb)
@@ -414,7 +410,7 @@ PVUSBURB vusbRhNewUrb(PVUSBROOTHUB pRh, uint8_t DstAddress, uint32_t cbData, uin
                                     +   sizeof(pUrb->Hci.paTds[0]) * cTdsAllocated);
         if (RT_UNLIKELY(!pUrb))
         {
-            RTCritSectLeave(&pRh->CritSect);
+            RTCritSectLeave(&pRh->CritSectFreeUrbs);
             AssertLogRelFailedReturn(NULL);
         }
 
@@ -426,7 +422,7 @@ PVUSBURB vusbRhNewUrb(PVUSBROOTHUB pRh, uint8_t DstAddress, uint32_t cbData, uin
         pUrb->VUsb.cTdsAllocated = cTdsAllocated;
         pUrb->Hci.paTds = (VUSBURB::VUSBURBHCI::VUSBURBHCITD *)(&pUrb->abData[cbDataAllocated + 16]);
     }
-    RTCritSectLeave(&pRh->CritSect);
+    RTCritSectLeave(&pRh->CritSectFreeUrbs);
 
     /*
      * (Re)init the URB
@@ -549,25 +545,35 @@ static DECLCALLBACK(int) vusbRhSubmitUrb(PVUSBIROOTHUBCONNECTOR pInterface, PVUS
 }
 
 
+static DECLCALLBACK(int) vusbRhReapAsyncUrbsWorker(PVUSBDEV pDev, RTMSINTERVAL cMillies)
+{
+    if (!cMillies)
+        vusbUrbDoReapAsync(pDev->pAsyncUrbHead, 0);
+    else
+    {
+        uint64_t u64Start = RTTimeMilliTS();
+        do
+        {
+            vusbUrbDoReapAsync(pDev->pAsyncUrbHead, RT_MIN(cMillies >> 8, 10));
+        } while (   pDev->pAsyncUrbHead
+                 && RTTimeMilliTS() - u64Start < cMillies);
+    }
+
+    return VINF_SUCCESS;
+}
+
 /** @copydoc VUSBIROOTHUBCONNECTOR::pfnReapAsyncUrbs */
-static DECLCALLBACK(void) vusbRhReapAsyncUrbs(PVUSBIROOTHUBCONNECTOR pInterface, RTMSINTERVAL cMillies)
+static DECLCALLBACK(void) vusbRhReapAsyncUrbs(PVUSBIROOTHUBCONNECTOR pInterface, PVUSBIDEVICE pDevice, RTMSINTERVAL cMillies)
 {
     PVUSBROOTHUB pRh = VUSBIROOTHUBCONNECTOR_2_VUSBROOTHUB(pInterface);
-    if (!pRh->pAsyncUrbHead)
+    PVUSBDEV pDev = (PVUSBDEV)pDevice;
+
+    if (!pDev->pAsyncUrbHead)
         return;
 
     STAM_PROFILE_START(&pRh->StatReapAsyncUrbs, a);
-    if (!cMillies)
-        vusbUrbDoReapAsync(pRh->pAsyncUrbHead, 0);
-    else
-    {
-        uint64_t    u64Start = RTTimeMilliTS();
-        do
-        {
-            vusbUrbDoReapAsync(pRh->pAsyncUrbHead, RT_MIN(cMillies >> 8, 10));
-        } while (   pRh->pAsyncUrbHead
-                 && RTTimeMilliTS() - u64Start < cMillies);
-    }
+    int rc = vusbDevIoThreadExecSync(pDev, (PFNRT)vusbRhReapAsyncUrbsWorker, 2, pDev, cMillies);
+    AssertRC(rc);
     STAM_PROFILE_STOP(&pRh->StatReapAsyncUrbs, a);
 }
 
@@ -584,18 +590,39 @@ static DECLCALLBACK(int) vusbRhCancelUrbsEp(PVUSBIROOTHUBCONNECTOR pInterface, P
      * Cancel and reap the URB(s) on an endpoint.
      */
     LogFlow(("vusbRhCancelUrbsEp: pRh=%p pUrb=%p\n", pRh));
-    vusbUrbCancel(pUrb, CANCELMODE_UNDO);
 
-    PVUSBURB pRipe;
-    if (pUrb->enmState == VUSBURBSTATE_REAPED)
-        pRipe = pUrb;
-    else
-        pRipe = pUrb->pUsbIns->pReg->pfnUrbReap(pUrb->pUsbIns, 0);
-    if (pRipe)
+    vusbUrbCancelAsync(pUrb, CANCELMODE_UNDO);
+
+    /* The reaper thread will take care of completing the URB. */
+
+    return VINF_SUCCESS;
+}
+
+/**
+ * Worker doing the actual cancelling of all outstanding URBs on the device I/O thread.
+ *
+ * @returns VBox status code.
+ * @param   pDev    USB device instance data.
+ */
+static DECLCALLBACK(int) vusbRhCancelAllUrbsWorker(PVUSBDEV pDev)
+{
+    /*
+     * Cancel the URBS.
+     *
+     * Not using th CritAsyncUrbs critical section here is safe
+     * as the I/O thread is the only thread accessing this struture at the
+     * moment.
+     */
+    PVUSBURB pUrb = pDev->pAsyncUrbHead;
+
+    while (pUrb)
     {
-        pRipe->enmStatus = VUSBSTATUS_CRC;
-        vusbUrbRipe(pRipe);
+        PVUSBURB pNext = pUrb->VUsb.pNext;
+        /* Call the worker directly. */
+        vusbUrbCancelWorker(pUrb, CANCELMODE_FAIL);
+        pUrb = pNext;
     }
+
     return VINF_SUCCESS;
 }
 
@@ -604,37 +631,14 @@ static DECLCALLBACK(void) vusbRhCancelAllUrbs(PVUSBIROOTHUBCONNECTOR pInterface)
 {
     PVUSBROOTHUB pRh = VUSBIROOTHUBCONNECTOR_2_VUSBROOTHUB(pInterface);
 
-    /*
-     * Cancel the URBS.
-     */
-    PVUSBURB pUrb = pRh->pAsyncUrbHead;
-    LogFlow(("vusbRhCancelAllUrbs: pRh=%p\n", pRh));
-    while (pUrb)
+    RTCritSectEnter(&pRh->CritSectDevices);
+    PVUSBDEV pDev = pRh->pDevices;
+    while (pDev)
     {
-        PVUSBURB pNext = pUrb->VUsb.pNext;
-        vusbUrbCancel(pUrb, CANCELMODE_FAIL);
-        pUrb = pNext;
-    }
-
-    /*
-     * Reap any URBs which now are ripe.
-     */
-    pUrb = pRh->pAsyncUrbHead;
-    while (pUrb)
-    {
-        PVUSBURB pRipe;
-        if (pUrb->enmState == VUSBURBSTATE_REAPED)
-            pRipe = pUrb;
-        else
-            pRipe = pUrb->pUsbIns->pReg->pfnUrbReap(pUrb->pUsbIns, 0);
-        if (!pRipe || pUrb == pRipe)
-            pUrb = pUrb->VUsb.pNext;
-        if (pRipe)
-        {
-            pRipe->enmStatus = VUSBSTATUS_CRC;
-            vusbUrbRipe(pRipe);
-        }
+        vusbDevIoThreadExecSync(pDev, (PFNRT)vusbRhCancelAllUrbsWorker, 1, pDev);
+        pDev = pDev->pNext;
     }
+    RTCritSectLeave(&pRh->CritSectDevices);
 }
 
 
@@ -703,7 +707,14 @@ static DECLCALLBACK(int) vusbRhDevPowerOff(PVUSBIDEVICE pInterface)
      * Cancel all URBs and reap them.
      */
     VUSBIRhCancelAllUrbs(&pRh->IRhConnector);
-    VUSBIRhReapAsyncUrbs(&pRh->IRhConnector, 0);
+    RTCritSectEnter(&pRh->CritSectDevices);
+    PVUSBDEV pDev = pRh->pDevices;
+    while (pDev)
+    {
+        VUSBIRhReapAsyncUrbs(&pRh->IRhConnector, (PVUSBIDEVICE)pDev, 0);
+        pDev = pDev->pNext;
+    }
+    RTCritSectLeave(&pRh->CritSectDevices);
 
     pRh->Hub.Dev.enmState = VUSB_DEVICE_STATE_ATTACHED;
     return VINF_SUCCESS;
@@ -756,8 +767,10 @@ static int vusbRhHubOpAttach(PVUSBHUB pHub, PVUSBDEV pDev)
     int rc = pRh->pIRhPort->pfnAttach(pRh->pIRhPort, &pDev->IDevice, iPort);
     if (RT_SUCCESS(rc))
     {
+        RTCritSectEnter(&pRh->CritSectDevices);
         pDev->pNext = pRh->pDevices;
         pRh->pDevices = pDev;
+        RTCritSectLeave(&pRh->CritSectDevices);
         LogRel(("VUSB: attached '%s' to port %d\n", pDev->pUsbIns->pszName, iPort));
     }
     else
@@ -786,6 +799,7 @@ static void vusbRhHubOpDetach(PVUSBHUB pHub, PVUSBDEV pDev)
     /*
      * Check that it's attached and unlink it from the linked list.
      */
+    RTCritSectEnter(&pRh->CritSectDevices);
     if (pRh->pDevices != pDev)
     {
         PVUSBDEV pPrev = pRh->pDevices;
@@ -797,6 +811,7 @@ static void vusbRhHubOpDetach(PVUSBHUB pHub, PVUSBDEV pDev)
     else
         pRh->pDevices = pDev->pNext;
     pDev->pNext = NULL;
+    RTCritSectLeave(&pRh->CritSectDevices);
 
     /*
      * Detach the device and mark the port as available.
@@ -872,7 +887,8 @@ static DECLCALLBACK(void) vusbRhDestruct(PPDMDRVINS pDrvIns)
         RTStrFree(pRh->Hub.pszName);
         pRh->Hub.pszName = NULL;
     }
-    RTCritSectDelete(&pRh->CritSect);
+    RTCritSectDelete(&pRh->CritSectDevices);
+    RTCritSectDelete(&pRh->CritSectFreeUrbs);
 }
 
 
@@ -901,9 +917,13 @@ static DECLCALLBACK(int) vusbRhConstruct(PPDMDRVINS pDrvIns, PCFGMNODE pCfg, uin
                     VERR_PDM_DRVINS_NO_ATTACH);
 
     /*
-     * Initialize the critical section.
+     * Initialize the critical sections.
      */
-    int rc = RTCritSectInit(&pThis->CritSect);
+    int rc = RTCritSectInit(&pThis->CritSectDevices);
+    if (RT_FAILURE(rc))
+        return rc;
+
+    rc = RTCritSectInit(&pThis->CritSectFreeUrbs);
     if (RT_FAILURE(rc))
         return rc;
 
