@@ -65,13 +65,14 @@
 #undef LOG_GROUP
 #include "../PC/DevLPC.cpp"
 #undef LOG_GROUP
-#include "../PC/DevSMC.cpp"
+#include "../EFI/DevSmc.cpp"
 #undef LOG_GROUP
 #include "../Storage/DevATA.cpp"
 #ifdef VBOX_WITH_USB
 # undef LOG_GROUP
 # include "../USB/DevOHCI.cpp"
 # ifdef VBOX_WITH_EHCI_IMPL
+#  undef LOG_GROUP
 #  include "../USB/DevEHCI.cpp"
 # endif
 #endif
