@@ -119,6 +119,10 @@ namespace UIDefs
     extern const char* GUI_HideDetails;
 
     /* Machine-window declarations: */
+#ifndef Q_WS_MAC
+    extern const char* GUI_MachineWindowIcons;
+    extern const char* GUI_MachineWindowNamePostfix;
+#endif /* !Q_WS_MAC */
     extern const char* GUI_RestrictedRuntimeMenus;
 #ifdef Q_WS_MAC
     extern const char* GUI_RestrictedRuntimeApplicationMenuActions;
@@ -140,6 +144,9 @@ namespace UIDefs
     extern const char* GUI_Fullscreen;
     extern const char* GUI_Seamless;
     extern const char* GUI_Scale;
+#ifdef Q_WS_X11
+    extern const char* GUI_Fullscreen_LegacyMode;
+#endif /* Q_WS_X11 */
     extern const char* GUI_VirtualScreenToHostScreen;
     extern const char* GUI_AutoresizeGuest;
     extern const char* GUI_AutomountGuestScreens;
@@ -148,6 +155,9 @@ namespace UIDefs
     extern const char* GUI_DefaultCloseAction;
     extern const char* GUI_RestrictedStatusBarIndicators;
     extern const char* GUI_HidLedsSync;
+    extern const char* GUI_MouseCapturePolicy;
+    extern const char* GUI_GuruMeditationHandler;
+    extern const char* GUI_HiDPIOptimization;
 
     /* Settings dialogs stuff: */
     extern const char* GUI_RestrictedGlobalSettingsPages;
@@ -227,7 +237,8 @@ enum MacOSXRelease
     MacOSXRelease_SnowLeopard,
     MacOSXRelease_Lion,
     MacOSXRelease_MountainLion,
-    MacOSXRelease_Mavericks
+    MacOSXRelease_Mavericks,
+    MacOSXRelease_Yosemite
 };
 #endif /* Q_WS_MAC */
 
@@ -469,5 +480,28 @@ enum MachineCloseAction
 };
 Q_DECLARE_METATYPE(MachineCloseAction);
 
+/** Mouse capture policy types. */
+enum MouseCapturePolicy
+{
+    MouseCapturePolicy_Default,
+    MouseCapturePolicy_HostComboOnly,
+    MouseCapturePolicy_Disabled
+};
+
+/** Guru Meditation handler types. */
+enum GuruMeditationHandlerType
+{
+    GuruMeditationHandlerType_Default,
+    GuruMeditationHandlerType_PowerOff,
+    GuruMeditationHandlerType_Ignore
+};
+
+/** Runtime UI: HiDPI optimization types. */
+enum HiDPIOptimizationType
+{
+    HiDPIOptimizationType_None,
+    HiDPIOptimizationType_Performance
+};
+
 #endif /* !___UIDefs_h___ */
 
