@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright (C) 2010-2012 Oracle Corporation
+ * Copyright (C) 2010-2013 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -132,13 +132,13 @@ STDMETHODIMP UIMainEventListener::HandleEvent(VBoxEventType_T /* type */, IEvent
         case KVBoxEventType_OnSnapshotTaken:
         {
             CSnapshotTakenEvent es(pEvent);
-            emit sigSnapshotChange(es.GetMachineId(), es.GetSnapshotId());
+            emit sigSnapshotTake(es.GetMachineId(), es.GetSnapshotId());
             break;
         }
         case KVBoxEventType_OnSnapshotDeleted:
         {
             CSnapshotDeletedEvent es(pEvent);
-            emit sigSnapshotChange(es.GetMachineId(), es.GetSnapshotId());
+            emit sigSnapshotDelete(es.GetMachineId(), es.GetSnapshotId());
             break;
         }
         case KVBoxEventType_OnSnapshotChanged:
@@ -162,7 +162,7 @@ STDMETHODIMP UIMainEventListener::HandleEvent(VBoxEventType_T /* type */, IEvent
         case KVBoxEventType_OnMouseCapabilityChanged:
         {
             CMouseCapabilityChangedEvent es(pEvent);
-            emit sigMouseCapabilityChange(es.GetSupportsAbsolute(), es.GetSupportsRelative(), es.GetNeedsHostCursor());
+            emit sigMouseCapabilityChange(es.GetSupportsAbsolute(), es.GetSupportsRelative(), es.GetSupportsMultiTouch(), es.GetNeedsHostCursor());
             break;
         }
         case KVBoxEventType_OnKeyboardLedsChanged:
@@ -208,6 +208,11 @@ STDMETHODIMP UIMainEventListener::HandleEvent(VBoxEventType_T /* type */, IEvent
             emit sigVRDEChange();
             break;
         }
+        case KVBoxEventType_OnVideoCaptureChanged:
+        {
+            emit sigVideoCaptureChange();
+            break;
+        }
         case KVBoxEventType_OnUSBControllerChanged:
         {
             emit sigUSBControllerChange();
