@@ -128,8 +128,6 @@ public:
     const char *vmRenderModeStr() const { return vm_render_mode_str; }
     bool isKWinManaged() const { return mIsKWinManaged; }
 
-    const QRect availableGeometry(int iScreen = 0) const;
-
     bool shouldRestoreCurrentSnapshot() const { return mRestoreCurrentSnapshot; }
     bool isPatmDisabled() const { return mDisablePatm; }
     bool isCsamDisabled() const { return mDisableCsam; }
@@ -313,6 +311,13 @@ public:
 
     static bool activateWindow (WId aWId, bool aSwitchDesktop = true);
 
+#ifdef Q_WS_X11
+    /** X11: Test whether the current window manager supports full screen mode. */
+    static bool supportsFullScreenMonitorsProtocolX11();
+    /** X11: Performs mapping of the passed @a pWidget to host-screen with passed @a uScreenId. */
+    static bool setFullScreenMonitorX11(QWidget *pWidget, ulong uScreenId);
+#endif /* Q_WS_X11 */
+
     static QString removeAccelMark (const QString &aText);
 
     static QString insertKeyToActionText (const QString &aText, const QString &aKey);
@@ -353,6 +358,9 @@ public:
 #ifdef VBOX_GUI_WITH_NETWORK_MANAGER
     static bool shouldWeAllowApplicationUpdate(CVirtualBox &vbox);
 #endif /* VBOX_GUI_WITH_NETWORK_MANAGER */
+#ifdef Q_WS_X11
+    static bool legacyFullscreenModeRequested(CVirtualBox vbox);
+#endif /* Q_WS_X11 */
     static bool shouldWeShowMachine(CMachine &machine);
     static bool shouldWeAllowMachineReconfiguration(CMachine &machine,
                                                     bool fIncludingMachineGeneralCheck = false,
@@ -378,6 +386,18 @@ public:
     static MachineCloseAction restrictedMachineCloseActions(CMachine &machine);
     static QList<GlobalSettingsPageType> restrictedGlobalSettingsPages(CVirtualBox &vbox);
     static QList<MachineSettingsPageType> restrictedMachineSettingsPages(CMachine &machine);
+#ifndef Q_WS_MAC
+    /** Except Mac OS X: Loads redefined machine-window icon names. */
+    static QStringList machineWindowIconNames(CMachine &machine);
+    /** Except Mac OS X: Loads redefined machine-window name postfix. */
+    static QString machineWindowNamePostfix(CMachine &machine);
+#endif /* !Q_WS_MAC */
+    /** Loads redefined mouse-capture policy type. */
+    static MouseCapturePolicy mouseCapturePolicy(CMachine &machine);
+    /** Loads redefined guru-meditation handler type. */
+    static GuruMeditationHandlerType guruMeditationHandlerType(CMachine &machine);
+    /** Loads Runtime UI HiDPI optimization type. */
+    static HiDPIOptimizationType hiDPIOptimizationType(CMachine &machine);
 
 #ifdef RT_OS_LINUX
     static void checkForWrongUSBMounted();
