@@ -33,6 +33,7 @@
 #include <iprt/initterm.h>
 #include <iprt/http.h>
 #include <iprt/err.h>
+#include <iprt/zip.h>
 
 /* Our network-reply thread: */
 class UINetworkReplyPrivateThread : public QThread
@@ -72,14 +73,17 @@ private:
     static int applyProxyRules(RTHTTP pHttp, const QString &strHostName, int iPort);
     static int applyCertificates(RTHTTP pHttp, const QString &strFullCertificateFileName);
     static int applyRawHeaders(RTHTTP pHttp, const QList<QByteArray> &headers, const QNetworkRequest &request);
-    static int performGetRequest(RTHTTP pHttp, const QNetworkRequest &request, QByteArray &reply);
+    static int performGetRequestForText(RTHTTP pHttp, const QNetworkRequest &request, QByteArray &reply);
+    static int performGetRequestForBinary(RTHTTP pHttp, const QNetworkRequest &request, QByteArray &reply);
     static int checkCertificates(RTHTTP pHttp, const QString &strFullCertificateFileName);
+    static int decompressCertificate(const QByteArray &package, QByteArray &certificate, const QString &strName);
     static int downloadCertificates(RTHTTP pHttp, const QString &strFullCertificateFileName);
-    static int downloadCertificatePca3G5(RTHTTP pHttp, QFile &file);
-    static int downloadCertificatePca3(RTHTTP pHttp, QFile &file);
+    static int downloadCertificatePca3G5(RTHTTP pHttp, QByteArray &certificate);
+    static int downloadCertificatePca3(RTHTTP pHttp, QByteArray &certificate);
     static int verifyCertificatePca3G5(RTHTTP pHttp, QByteArray &certificate);
     static int verifyCertificatePca3(RTHTTP pHttp, QByteArray &certificate);
     static int verifyCertificate(RTHTTP pHttp, QByteArray &certificate, const QByteArray &sha1, const QByteArray &sha512);
+    static int saveCertificates(const QString &strFullCertificateFileName, const QByteArray &certificatePca3G5, const QByteArray &certificatePca3);
     static int saveCertificate(QFile &file, const QByteArray &certificate);
 
     /* Variables: */
@@ -157,7 +161,7 @@ int UINetworkReplyPrivateThread::applyRawHeaders()
 int UINetworkReplyPrivateThread::performMainRequest()
 {
     /* Perform GET request: */
-    return performGetRequest(m_pHttp, m_request, m_reply);
+    return performGetRequestForText(m_pHttp, m_request, m_reply);
 }
 
 void UINetworkReplyPrivateThread::run()
@@ -260,19 +264,37 @@ int UINetworkReplyPrivateThread::applyRawHeaders(RTHTTP pHttp, const QList<QByte
 }
 
 /* static */
-int UINetworkReplyPrivateThread::performGetRequest(RTHTTP pHttp, const QNetworkRequest &request, QByteArray &reply)
+int UINetworkReplyPrivateThread::performGetRequestForText(RTHTTP pHttp, const QNetworkRequest &request, QByteArray &reply)
 {
     /* Make sure HTTP is created: */
     if (!pHttp)
         return VERR_INVALID_POINTER;
 
     /* Perform blocking HTTP GET request: */
-    char *pszBuf = 0;
-    int rc = RTHttpGet(pHttp,
-                       request.url().toString().toAscii().constData(),
-                       &pszBuf);
-    reply = QByteArray(pszBuf);
-    RTMemFree(pszBuf);
+    char *pszBuffer = 0;
+    int rc = RTHttpGetText(pHttp,
+                           request.url().toString().toAscii().constData(),
+                           &pszBuffer);
+    reply = QByteArray(pszBuffer);
+    RTMemFree(pszBuffer);
+    return rc;
+}
+
+/* static */
+int UINetworkReplyPrivateThread::performGetRequestForBinary(RTHTTP pHttp, const QNetworkRequest &request, QByteArray &reply)
+{
+    /* Make sure HTTP is created: */
+    if (!pHttp)
+        return VERR_INVALID_POINTER;
+
+    /* Perform blocking HTTP GET request: */
+    void *pBuffer = 0;
+    size_t size = 0;
+    int rc = RTHttpGetBinary(pHttp,
+                             request.url().toString().toAscii().constData(),
+                             &pBuffer, &size);
+    reply = QByteArray((const char*)pBuffer, (int)size);
+    RTMemFree(pBuffer);
     return rc;
 }
 
@@ -333,65 +355,101 @@ int UINetworkReplyPrivateThread::checkCertificates(RTHTTP pHttp, const QString &
     return rc;
 }
 
+/* static */
+int UINetworkReplyPrivateThread::decompressCertificate(const QByteArray &package, QByteArray &certificate, const QString &strName)
+{
+    /* Decompress certificate: */
+    void *pDecompressedBuffer;
+    size_t cDecompressedSize;
+    int rc = RTZipPkzipMemDecompress(&pDecompressedBuffer, &cDecompressedSize, package, package.size(), strName.toLatin1().constData());
+    if (RT_SUCCESS(rc))
+    {
+        /* Copy certificate: */
+        certificate = QByteArray((const char*)pDecompressedBuffer, (int)cDecompressedSize);
+        /* Free decompressed buffer: */
+        RTMemFree(pDecompressedBuffer);
+    }
+    /* Return result: */
+    return rc;
+}
+
 /* static */
 int UINetworkReplyPrivateThread::downloadCertificates(RTHTTP pHttp, const QString &strFullCertificateFileName)
 {
-    /* Open certificates file: */
-    QFile file(strFullCertificateFileName);
-    bool fFileOpened = file.open(QIODevice::WriteOnly);
-    int rc = fFileOpened ? VINF_SUCCESS : VERR_OPEN_FAILED;
+    /* Prepare certificates: */
+    QByteArray certificatePca3G5;
+    QByteArray certificatePca3;
 
-    /* Download PCA-3G5 certificate: */
+    /* Receive certificate package: */
+    QByteArray package;
+    const QNetworkRequest address(QUrl("http://www.verisign.com/support/roots.zip"));
+    int rc = performGetRequestForBinary(pHttp, address, package);
+    /* UnZIP PCA-3G5 certificate: */
     if (RT_SUCCESS(rc))
-        rc = downloadCertificatePca3G5(pHttp, file);
-    /* Download PCA-3 certificate: */
+    {
+        rc = decompressCertificate(package, certificatePca3G5,
+                                   "VeriSign Root Certificates/Generation 5 (G5) PCA/VeriSign Class 3 Public Primary Certification Authority - G5.pem");
+        /* Verify PCA-3G5 certificate: */
+        if (RT_SUCCESS(rc))
+            rc = verifyCertificatePca3G5(pHttp, certificatePca3G5);
+    }
+    /* UnZIP PCA-3 certificate: */
     if (RT_SUCCESS(rc))
-        rc = downloadCertificatePca3(pHttp, file);
+    {
+        rc = decompressCertificate(package, certificatePca3,
+                                   "VeriSign Root Certificates/Generation 1 (G1) PCAs/Class 3 Public Primary Certification Authority.pem");
+        /* Verify PCA-3 certificate: */
+        if (RT_SUCCESS(rc))
+            rc = verifyCertificatePca3(pHttp, certificatePca3);
+    }
 
-    /* Close certificates file: */
-    if (fFileOpened)
-        file.close();
+    /* Fallback.. download certificates separately: */
+    if (!RT_SUCCESS(rc))
+    {
+        /* Reset result: */
+        rc = VINF_SUCCESS;
+        /* Download PCA-3G5 certificate: */
+        if (RT_SUCCESS(rc))
+            rc = downloadCertificatePca3G5(pHttp, certificatePca3G5);
+        /* Download PCA-3 certificate: */
+        if (RT_SUCCESS(rc))
+            rc = downloadCertificatePca3(pHttp, certificatePca3);
+    }
+
+    /* Save certificates: */
+    if (RT_SUCCESS(rc))
+        saveCertificates(strFullCertificateFileName, certificatePca3G5, certificatePca3);
 
     /* Return result-code: */
     return rc;
 }
 
 /* static */
-int UINetworkReplyPrivateThread::downloadCertificatePca3G5(RTHTTP pHttp, QFile &file)
+int UINetworkReplyPrivateThread::downloadCertificatePca3G5(RTHTTP pHttp, QByteArray &certificate)
 {
     /* Receive certificate: */
-    QByteArray certificate;
     const QNetworkRequest address(QUrl("http://www.verisign.com/repository/roots/root-certificates/PCA-3G5.pem"));
-    int rc = performGetRequest(pHttp, address, certificate);
+    int rc = performGetRequestForText(pHttp, address, certificate);
 
     /* Verify certificate: */
     if (RT_SUCCESS(rc))
         rc = verifyCertificatePca3G5(pHttp, certificate);
 
-    /* Save certificate: */
-    if (RT_SUCCESS(rc))
-        rc = saveCertificate(file, certificate);
-
     /* Return result-code: */
     return rc;
 }
 
 /* static */
-int UINetworkReplyPrivateThread::downloadCertificatePca3(RTHTTP pHttp, QFile &file)
+int UINetworkReplyPrivateThread::downloadCertificatePca3(RTHTTP pHttp, QByteArray &certificate)
 {
     /* Receive certificate: */
-    QByteArray certificate;
     const QNetworkRequest address(QUrl("http://www.verisign.com/repository/roots/root-certificates/PCA-3.pem"));
-    int rc = performGetRequest(pHttp, address, certificate);
+    int rc = performGetRequestForText(pHttp, address, certificate);
 
     /* Verify certificate: */
     if (RT_SUCCESS(rc))
         rc = verifyCertificatePca3(pHttp, certificate);
 
-    /* Save certificate: */
-    if (RT_SUCCESS(rc))
-        rc = saveCertificate(file, certificate);
-
     /* Return result-code: */
     return rc;
 }
@@ -485,6 +543,30 @@ int UINetworkReplyPrivateThread::verifyCertificate(RTHTTP pHttp, QByteArray &cer
     return rc;
 }
 
+/* static */
+int UINetworkReplyPrivateThread::saveCertificates(const QString &strFullCertificateFileName,
+                                                  const QByteArray &certificatePca3G5,
+                                                  const QByteArray &certificatePca3)
+{
+    /* Open certificates file: */
+    QFile file(strFullCertificateFileName);
+    bool fFileOpened = file.open(QIODevice::WriteOnly);
+    int rc = fFileOpened ? VINF_SUCCESS : VERR_OPEN_FAILED;
+
+    /* Save certificates: */
+    if (RT_SUCCESS(rc))
+        rc = saveCertificate(file, certificatePca3G5);
+    if (RT_SUCCESS(rc))
+        rc = saveCertificate(file, certificatePca3);
+
+    /* Close certificates file: */
+    if (fFileOpened)
+        file.close();
+
+    /* Return result-code: */
+    return rc;
+}
+
 /* static */
 int UINetworkReplyPrivateThread::saveCertificate(QFile &file, const QByteArray &certificate)
 {
