@@ -203,8 +203,9 @@ void UIMachineView::sltPerformGuestResize(const QSize &toSize)
     setMaxGuestSize(newSize);
 
     /* Send new size-hint to the guest: */
-    LogRelFlow(("UIMachineView: Sending guest size-hint to screen %d: %dx%d\n",
-                (int)screenId(), newSize.width(), newSize.height()));
+    LogRel(("UIMachineView::sltPerformGuestResize: "
+            "Sending guest size-hint to screen %d as %dx%d\n",
+            (int)screenId(), newSize.width(), newSize.height()));
     session().GetConsole().GetDisplay().SetVideoModeHint(screenId(),
                                                          uisession()->isScreenVisible(screenId()),
                                                          false, 0, 0, newSize.width(), newSize.height(), 0);
@@ -437,6 +438,7 @@ void UIMachineView::prepareFrameBuffer()
 # else /* VBOX_WITH_VIDEOHWACCEL */
                 pFrameBuffer = new UIFrameBufferQImage(this);
 # endif /* !VBOX_WITH_VIDEOHWACCEL */
+                pFrameBuffer->setHiDPIOptimizationType(uisession()->hiDPIOptimizationType());
                 uisession()->setFrameBuffer(screenId(), pFrameBuffer);
             }
             m_pFrameBuffer = pFrameBuffer;
@@ -772,6 +774,9 @@ void UIMachineView::storeGuestSizeHint(const QSize &sizeHint)
     CMachine machine = session().GetMachine();
 
     /* Save machine view hint: */
+    LogRel(("UIMachineView::storeGuestSizeHint: "
+            "Storing guest size-hint for screen %d as %dx%d\n",
+            (int)screenId(), sizeHint.width(), sizeHint.height()));
     QString strKey = makeExtraDataKeyPerMonitor(GUI_LastGuestSizeHint);
     QString strValue = QString("%1,%2").arg(sizeHint.width()).arg(sizeHint.height());
     machine.SetExtraData(strKey, strValue);
@@ -1033,6 +1038,15 @@ bool UIMachineView::eventFilter(QObject *pWatched, QEvent *pEvent)
                 }
                 break;
             }
+#ifdef Q_WS_MAC
+            case QEvent::Move:
+            {
+                /* Update backing scale factor for underlying frame-buffer: */
+                if (m_pFrameBuffer)
+                    m_pFrameBuffer->setBackingScaleFactor(darwinBackingScaleFactor(machineWindow()));
+                break;
+            }
+#endif /* Q_WS_MAC */
             default:
                 break;
         }
