@@ -40,6 +40,9 @@ class CSession;
 class CUSBDevice;
 class CNetworkAdapter;
 class CMediumAttachment;
+#ifndef Q_WS_MAC
+class QIcon;
+#endif /* !Q_WS_MAC */
 
 /* CConsole callback event types: */
 enum UIConsoleEventType
@@ -93,6 +96,26 @@ public:
     QMenuBar* newMenuBar(RuntimeMenuType fOptions = RuntimeMenuType_All);
     QCursor cursor() const { return m_cursor; }
 
+#ifndef Q_WS_MAC
+    /** @name Branding stuff.
+     ** @{ */
+    /** Returns redefined machine-window icon. */
+    QIcon* machineWindowIcon() const { return m_pMachineWindowIcon; }
+    /** Returns redefined machine-window name postfix. */
+    QString machineWindowNamePostfix() const { return m_strMachineWindowNamePostfix; }
+    /** @} */
+#endif /* !Q_WS_MAC */
+
+    /** @name Runtime workflow stuff.
+     ** @{ */
+    /** Returns the mouse-capture policy. */
+    MouseCapturePolicy mouseCapturePolicy() const { return m_mouseCapturePolicy; }
+    /** Returns Guru Meditation handler type. */
+    GuruMeditationHandlerType guruMeditationHandlerType() const { return m_guruMeditationHandlerType; }
+    /** Returns HiDPI optimization type. */
+    HiDPIOptimizationType hiDPIOptimizationType() const { return m_hiDPIOptimizationType; }
+    /** @} */
+
     /** @name Extension Pack stuff.
      ** @{ */
     /** Determines whether extension pack installed and usable. */
@@ -159,6 +182,7 @@ public:
     bool isStuck() const { return machineState() == KMachineState_Stuck; }
     bool wasPaused() const { return machineStatePrevious() == KMachineState_Paused ||
                                     machineStatePrevious() == KMachineState_TeleportingPausedVM; }
+    bool isStarted() const { return m_fIsStarted; }
     bool isFirstTimeStarted() const { return m_fIsFirstTimeStarted; }
     bool isIgnoreRuntimeMediumsChanging() const { return m_fIsIgnoreRuntimeMediumsChanging; }
     bool isGuestResizeIgnored() const { return m_fIsGuestResizeIgnored; }
@@ -242,12 +266,15 @@ signals:
     void sigCPUExecutionCapChange();
     void sigGuestMonitorChange(KGuestMonitorChangedEventType changeType, ulong uScreenId, QRect screenGeo);
 
-    /* Notifiers: Qt callback stuff: */
-    void sigHostScreenCountChanged();
-    void sigHostScreenGeometryChanged();
+    /** Notifies about host-screen count change. */
+    void sigHostScreenCountChange();
+    /** Notifies about host-screen geometry change. */
+    void sigHostScreenGeometryChange();
+    /** Notifies about host-screen available-area change. */
+    void sigHostScreenAvailableAreaChange();
 
     /* Session signals: */
-    void sigMachineStarted();
+    void sigStarted();
 
 public slots:
 
@@ -255,6 +282,9 @@ public slots:
 
 private slots:
 
+    /** Marks machine started. */
+    void sltMarkStarted() { m_fIsStarted = true; }
+
     /* Handler: Close Runtime UI stuff: */
     void sltCloseRuntimeUI();
 
@@ -273,8 +303,13 @@ private slots:
     void sltHandleHostDisplayAboutToChange();
     void sltCheckIfHostDisplayChanged();
 #endif /* RT_OS_DARWIN */
+
+    /** Handles host-screen count change. */
     void sltHandleHostScreenCountChange();
+    /** Handles host-screen geometry change. */
     void sltHandleHostScreenGeometryChange();
+    /** Handles host-screen available-area change. */
+    void sltHandleHostScreenAvailableAreaChange();
 
 private:
 
@@ -329,6 +364,26 @@ private:
     KMachineState m_machineState;
     QCursor m_cursor;
 
+#ifndef Q_WS_MAC
+    /** @name Branding variables.
+     ** @{ */
+    /** Holds redefined machine-window icon. */
+    QIcon *m_pMachineWindowIcon;
+    /** Holds redefined machine-window name postfix. */
+    QString m_strMachineWindowNamePostfix;
+    /** @} */
+#endif /* !Q_WS_MAC */
+
+    /** @name Runtime workflow variables.
+     ** @{ */
+    /** Holds the mouse-capture policy. */
+    MouseCapturePolicy m_mouseCapturePolicy;
+    /** Holds Guru Meditation handler type. */
+    GuruMeditationHandlerType m_guruMeditationHandlerType;
+    /** Holds HiDPI optimization type. */
+    HiDPIOptimizationType m_hiDPIOptimizationType;
+    /** @} */
+
     /** @name Extension Pack variables.
      ** @{ */
     /** Determines whether extension pack installed and usable. */
@@ -392,6 +447,7 @@ private:
     /** @} */
 
     /* Common flags: */
+    bool m_fIsStarted : 1;
     bool m_fIsFirstTimeStarted : 1;
     bool m_fIsIgnoreRuntimeMediumsChanging : 1;
     bool m_fIsGuestResizeIgnored : 1;
