@@ -27,7 +27,9 @@
 #include "UIMachineWindowFullscreen.h"
 #include "UIMultiScreenLayout.h"
 #ifdef Q_WS_MAC
+# include "UIMachineView.h"
 # include "UIExtraDataEventHandler.h"
+# include "UICocoaApplication.h"
 # include "VBoxUtils.h"
 # include "UIFrameBuffer.h"
 # include <Carbon/Carbon.h>
@@ -36,6 +38,7 @@
 UIMachineLogicFullscreen::UIMachineLogicFullscreen(QObject *pParent, UISession *pSession)
     : UIMachineLogic(pParent, pSession, UIVisualStateType_Fullscreen)
 #ifdef Q_WS_MAC
+    , m_fScreensHaveSeparateSpaces(darwinScreensHaveSeparateSpaces())
     , m_fadeToken(kCGDisplayFadeReservationInvalidToken)
 #endif /* Q_WS_MAC */
 {
@@ -80,14 +83,28 @@ bool UIMachineLogicFullscreen::checkAvailability()
     return true;
 }
 
-/** Adjusts guest screen count/size for the machine-logic we have. */
-void UIMachineLogicFullscreen::maybeAdjustGuestScreenSize()
+void UIMachineLogicFullscreen::adjustMachineWindowsGeometry()
 {
-    /* We should rebuild screen-layout: */
+    LogRel(("UIMachineLogicFullscreen::adjustMachineWindowsGeometry\n"));
+
+    /* Rebuild multi-screen layout: */
     m_pScreenLayout->rebuild();
+
+#ifdef Q_WS_MAC
+    /* For Lion and previous: */
+    if (vboxGlobal().osRelease() <= MacOSXRelease_Lion)
+    {
+        /* Make sure all machine-window(s) have proper geometry: */
+        foreach (UIMachineWindow *pMachineWindow, machineWindows())
+            pMachineWindow->showInNecessaryMode();
+    }
+    /* For ML and next revalidate native fullscreen: */
+    else revalidateNativeFullScreen();
+#else /* !Q_WS_MAC */
     /* Make sure all machine-window(s) have proper geometry: */
     foreach (UIMachineWindow *pMachineWindow, machineWindows())
         pMachineWindow->showInNecessaryMode();
+#endif /* !Q_WS_MAC */
 }
 
 int UIMachineLogicFullscreen::hostScreenForGuestScreen(int iScreenId) const
@@ -108,7 +125,10 @@ void UIMachineLogicFullscreen::sltHandleNativeFullscreenWillEnter()
 
     /* Get sender machine-window: */
     UIMachineWindow *pMachineWindow = qobject_cast<UIMachineWindow*>(sender());
-    AssertReturnVoid(pMachineWindow);
+    AssertPtrReturnVoid(pMachineWindow);
+    LogRel(("UIMachineLogicFullscreen::sltHandleNativeFullscreenWillEnter: "
+            "Machine-window #%d will enter native fullscreen.\n",
+            (int)pMachineWindow->screenId()));
 
     /* Fade to black: */
     fadeToBlack();
@@ -121,20 +141,22 @@ void UIMachineLogicFullscreen::sltHandleNativeFullscreenDidEnter()
 
     /* Get sender machine-window: */
     UIMachineWindow *pMachineWindow = qobject_cast<UIMachineWindow*>(sender());
-    AssertReturnVoid(pMachineWindow);
+    AssertPtrReturnVoid(pMachineWindow);
+    LogRel(("UIMachineLogicFullscreen::sltHandleNativeFullscreenDidEnter: "
+            "Machine-window #%d did enter native fullscreen.\n",
+            (int)pMachineWindow->screenId()));
 
     /* Add machine-window to corresponding set: */
     m_fullscreenMachineWindows.insert(pMachineWindow);
     AssertReturnVoid(m_fullscreenMachineWindows.contains(pMachineWindow));
 
-    /* Fade to normal if necessary: */
-    QSet<UIMachineWindow*> visibleMachineWindows;
-    foreach (UIMachineWindow *pMachineWindow, machineWindows())
-        if (uisession()->isScreenVisible(pMachineWindow->screenId()))
-            visibleMachineWindows << pMachineWindow;
-    if (   !darwinScreensHaveSeparateSpaces()
-        || m_fullscreenMachineWindows == visibleMachineWindows)
-        fadeToNormal();
+    /* Fade to normal: */
+    fadeToNormal();
+
+    /* Rebuild multi-screen layout: */
+    m_pScreenLayout->rebuild();
+    /* Revalidate native fullscreen: */
+    revalidateNativeFullScreen();
 }
 
 void UIMachineLogicFullscreen::sltHandleNativeFullscreenWillExit()
@@ -144,7 +166,10 @@ void UIMachineLogicFullscreen::sltHandleNativeFullscreenWillExit()
 
     /* Get sender machine-window: */
     UIMachineWindow *pMachineWindow = qobject_cast<UIMachineWindow*>(sender());
-    AssertReturnVoid(pMachineWindow);
+    AssertPtrReturnVoid(pMachineWindow);
+    LogRel(("UIMachineLogicFullscreen::sltHandleNativeFullscreenWillExit: "
+            "Machine-window #%d will exit native fullscreen.\n",
+            (int)pMachineWindow->screenId()));
 
     /* Fade to black: */
     fadeToBlack();
@@ -157,25 +182,33 @@ void UIMachineLogicFullscreen::sltHandleNativeFullscreenDidExit()
 
     /* Get sender machine-window: */
     UIMachineWindow *pMachineWindow = qobject_cast<UIMachineWindow*>(sender());
-    AssertReturnVoid(pMachineWindow);
+    AssertPtrReturnVoid(pMachineWindow);
 
     /* Remove machine-window from corresponding set: */
     bool fResult = m_fullscreenMachineWindows.remove(pMachineWindow);
-    AssertReturnVoid(fResult && !m_fullscreenMachineWindows.contains(pMachineWindow));
-    Q_UNUSED(fResult);
+    AssertReturnVoid(!m_fullscreenMachineWindows.contains(pMachineWindow));
+
+    /* We have same signal if window did fail to enter native fullscreen.
+     * In that case window missed in m_fullscreenMachineWindows,
+     * ignore this signal silently: */
+    if (!fResult)
+        return;
 
     /* If that window was invalidated: */
     if (m_invalidFullscreenMachineWindows.contains(pMachineWindow))
     {
         LogRel(("UIMachineLogicFullscreen::sltHandleNativeFullscreenDidExit: "
-                "Machine-window #%d exited invalidated fullscreen, revalidate it.\n",
+                "Machine-window #%d exited invalidated native fullscreen, revalidate it.\n",
                 (int)pMachineWindow->screenId()));
 
-        /* Exclude window from invalidation list: */
+        /* Exclude machine-window from invalidation set: */
         m_invalidFullscreenMachineWindows.remove(pMachineWindow);
+        AssertReturnVoid(!m_invalidFullscreenMachineWindows.contains(pMachineWindow));
 
-        /* Revalidate 'fullscreen' window: */
-        revalidateFullscreenWindow(pMachineWindow);
+        /* Rebuild multi-screen layout: */
+        m_pScreenLayout->rebuild();
+        /* Revalidate native fullscreen: */
+        revalidateNativeFullScreen();
     }
     /* If there are no invalidated windows: */
     else if (m_invalidFullscreenMachineWindows.isEmpty())
@@ -184,7 +217,8 @@ void UIMachineLogicFullscreen::sltHandleNativeFullscreenDidExit()
         if (!m_fullscreenMachineWindows.isEmpty())
         {
             LogRel(("UIMachineLogicFullscreen::sltHandleNativeFullscreenDidExit: "
-                    "Machine-window exited fullscreen, asking others to exit too...\n"));
+                    "Machine-window #%d exited native fullscreen, asking others to exit too...\n",
+                    (int)pMachineWindow->screenId()));
 
             /* Ask window(s) to exit 'fullscreen' mode: */
             emit sigNotifyAboutNativeFullscreenShouldBeExited();
@@ -193,7 +227,8 @@ void UIMachineLogicFullscreen::sltHandleNativeFullscreenDidExit()
         else
         {
             LogRel(("UIMachineLogicFullscreen::sltHandleNativeFullscreenDidExit: "
-                    "Machine-window(s) exited fullscreen, changing visual-state to requested...\n"));
+                    "Machine-window #%d exited native fullscreen, changing visual-state to requested...\n",
+                    (int)pMachineWindow->screenId()));
 
             /* Change visual-state to requested: */
             UIVisualStateType type = uisession()->requestedVisualState();
@@ -208,6 +243,45 @@ void UIMachineLogicFullscreen::sltHandleNativeFullscreenDidExit()
     }
 }
 
+void UIMachineLogicFullscreen::sltHandleNativeFullscreenFailToEnter()
+{
+    /* Make sure this method is only used for ML and next: */
+    AssertReturnVoid(vboxGlobal().osRelease() > MacOSXRelease_Lion);
+
+    /* Get sender machine-window: */
+    UIMachineWindow *pMachineWindow = qobject_cast<UIMachineWindow*>(sender());
+    AssertReturnVoid(pMachineWindow);
+
+    /* Make sure this window is not registered somewhere: */
+    AssertReturnVoid(!m_fullscreenMachineWindows.remove(pMachineWindow));
+    AssertReturnVoid(!m_invalidFullscreenMachineWindows.remove(pMachineWindow));
+
+    /* If there are 'fullscreen' windows: */
+    if (!m_fullscreenMachineWindows.isEmpty())
+    {
+        LogRel(("UIMachineLogicFullscreen::sltHandleNativeFullscreenFailToEnter: "
+                "Machine-window #%d failed to enter native fullscreen, asking others to exit...\n",
+                (int)pMachineWindow->screenId()));
+
+        /* Ask window(s) to exit 'fullscreen' mode: */
+        emit sigNotifyAboutNativeFullscreenShouldBeExited();
+    }
+    /* If there are no 'fullscreen' windows: */
+    else
+    {
+        LogRel(("UIMachineLogicFullscreen::sltHandleNativeFullscreenFailToEnter: "
+                "Machine-window #%d failed to enter native fullscreen, requesting change visual-state to normal...\n",
+                (int)pMachineWindow->screenId()));
+
+        /* Ask session to change 'fullscreen' mode to 'normal': */
+        uisession()->setRequestedVisualState(UIVisualStateType_Normal);
+
+        /* If session started already => push mode-change directly: */
+        if (uisession()->isStarted())
+            sltCheckForRequestedVisualStateType();
+    }
+}
+
 void UIMachineLogicFullscreen::sltChangeVisualStateToNormal()
 {
     /* Base-class handling for Lion and previous: */
@@ -252,6 +326,29 @@ void UIMachineLogicFullscreen::sltChangeVisualStateToScale()
         emit sigNotifyAboutNativeFullscreenShouldBeExited();
     }
 }
+
+void UIMachineLogicFullscreen::sltCheckForRequestedVisualStateType()
+{
+    /* Do not try to change visual-state type if machine was not started yet: */
+    if (!uisession()->isRunning() && !uisession()->isPaused())
+        return;
+
+    /* Check requested visual-state types: */
+    switch (uisession()->requestedVisualState())
+    {
+        /* If 'normal' visual-state type is requested: */
+        case UIVisualStateType_Normal:
+        {
+            LogRel(("UIMachineLogicFullscreen::sltCheckForRequestedVisualStateType: "
+                    "Going 'normal' as requested...\n"));
+            uisession()->setRequestedVisualState(UIVisualStateType_Invalid);
+            uisession()->changeVisualState(UIVisualStateType_Normal);
+            break;
+        }
+        default:
+            break;
+    }
+}
 #endif /* RT_OS_DARWIN */
 
 void UIMachineLogicFullscreen::sltMachineStateChanged()
@@ -262,17 +359,14 @@ void UIMachineLogicFullscreen::sltMachineStateChanged()
     /* If machine-state changed from 'paused' to 'running': */
     if (uisession()->isRunning() && uisession()->wasPaused())
     {
-        LogRelFlow(("UIMachineLogicFullscreen: "
-                    "Machine-state changed from 'paused' to 'running': "
-                    "Updating screen-layout...\n"));
+        LogRel(("UIMachineLogicFullscreen::sltMachineStateChanged:"
+                "Machine-state changed from 'paused' to 'running': "
+                "Adjust machine-window geometry...\n"));
 
         /* Make sure further code will be called just once: */
         uisession()->forgetPreviousMachineState();
-        /* We should rebuild screen-layout: */
-        m_pScreenLayout->rebuild();
-        /* Make sure all machine-window(s) have proper geometry: */
-        foreach (UIMachineWindow *pMachineWindow, machineWindows())
-            pMachineWindow->showInNecessaryMode();
+        /* Adjust machine-window geometry if necessary: */
+        adjustMachineWindowsGeometry();
     }
 }
 
@@ -285,7 +379,7 @@ void UIMachineLogicFullscreen::sltChangePresentationMode(bool /* fEnabled */)
 
 void UIMachineLogicFullscreen::sltScreenLayoutChanged()
 {
-    LogRel(("UIMachineLogicFullscreen: Multi-screen layout changed.\n"));
+    LogRel(("UIMachineLogicFullscreen::sltScreenLayoutChanged: Multi-screen layout changed.\n"));
 
 #ifdef Q_WS_MAC
     /* For Lion and previous: */
@@ -297,8 +391,8 @@ void UIMachineLogicFullscreen::sltScreenLayoutChanged()
         /* Update 'presentation mode': */
         setPresentationModeEnabled(true);
     }
-    /* Revalidate 'fullscreen' windows for ML and next: */
-    else revalidateFullscreenWindows();
+    /* Revalidate native fullscreen for ML and next: */
+    else revalidateNativeFullScreen();
 #else /* !Q_WS_MAC */
     /* Make sure all machine-window(s) have proper geometry: */
     foreach (UIMachineWindow *pMachineWindow, machineWindows())
@@ -310,42 +404,45 @@ void UIMachineLogicFullscreen::sltGuestMonitorChange(KGuestMonitorChangedEventTy
 {
     LogRel(("UIMachineLogicFullscreen: Guest-screen count changed.\n"));
 
-    /* Update multi-screen layout before any window update: */
-    if (changeType == KGuestMonitorChangedEventType_Enabled ||
-        changeType == KGuestMonitorChangedEventType_Disabled)
-        m_pScreenLayout->rebuild();
+    /* Rebuild multi-screen layout: */
+    m_pScreenLayout->rebuild();
 
 #ifdef Q_WS_MAC
     /* Call to base-class for Lion and previous: */
     if (vboxGlobal().osRelease() <= MacOSXRelease_Lion)
         UIMachineLogic::sltGuestMonitorChange(changeType, uScreenId, screenGeo);
-    /* Revalidate 'fullscreen' windows for ML and next: */
-    else revalidateFullscreenWindows();
+    /* Revalidate native fullscreen for ML and next: */
+    else revalidateNativeFullScreen();
 #else /* !Q_WS_MAC */
     /* Call to base-class: */
     UIMachineLogic::sltGuestMonitorChange(changeType, uScreenId, screenGeo);
 #endif /* !Q_WS_MAC */
 }
 
-void UIMachineLogicFullscreen::sltHostScreenCountChanged()
+void UIMachineLogicFullscreen::sltHostScreenCountChange()
 {
     LogRel(("UIMachineLogicFullscreen: Host-screen count changed.\n"));
 
-    /* Update multi-screen layout before any window update: */
+    /* Rebuild multi-screen layout: */
     m_pScreenLayout->rebuild();
 
 #ifdef Q_WS_MAC
     /* Call to base-class for Lion and previous: */
     if (vboxGlobal().osRelease() <= MacOSXRelease_Lion)
-        UIMachineLogic::sltHostScreenCountChanged();
-    /* Revalidate 'fullscreen' windows for ML and next: */
-    else revalidateFullscreenWindows();
+        UIMachineLogic::sltHostScreenCountChange();
+    /* Revalidate native fullscreen for ML and next: */
+    else revalidateNativeFullScreen();
 #else /* !Q_WS_MAC */
     /* Call to base-class: */
-    UIMachineLogic::sltHostScreenCountChanged();
+    UIMachineLogic::sltHostScreenCountChange();
 #endif /* !Q_WS_MAC */
 }
 
+void UIMachineLogicFullscreen::sltHostScreenAvailableAreaChange()
+{
+    LogRel(("UIMachineLogicFullscreen: Host-screen available-area change ignored.\n"));
+}
+
 void UIMachineLogicFullscreen::prepareActionGroups()
 {
     /* Call to base-class: */
@@ -397,6 +494,16 @@ void UIMachineLogicFullscreen::prepareMachineWindows()
         return;
 
 #ifdef Q_WS_MAC
+    /* For ML and next: */
+    if (vboxGlobal().osRelease() > MacOSXRelease_Lion)
+    {
+        /* Register to native notifications: */
+        UICocoaApplication::instance()->registerToNotificationOfWorkspace("NSWorkspaceDidActivateApplicationNotification", this,
+                                                                          UIMachineLogicFullscreen::nativeHandlerForApplicationActivation);
+        UICocoaApplication::instance()->registerToNotificationOfWorkspace("NSWorkspaceActiveSpaceDidChangeNotification", this,
+                                                                          UIMachineLogicFullscreen::nativeHandlerForActiveSpaceChange);
+    }
+
     /* We have to make sure that we are getting the front most process.
      * This is necessary for Qt versions > 4.3.3: */
     darwinSetFrontMostProcess();
@@ -437,16 +544,23 @@ void UIMachineLogicFullscreen::prepareMachineWindows()
                     pMachineWindow, SLOT(sltExitNativeFullscreen(UIMachineWindow*)));
             /* Window => logic signals: */
             connect(pMachineWindow, SIGNAL(sigNotifyAboutNativeFullscreenWillEnter()),
-                    this, SLOT(sltHandleNativeFullscreenWillEnter()));
+                    this, SLOT(sltHandleNativeFullscreenWillEnter()),
+                    Qt::QueuedConnection);
             connect(pMachineWindow, SIGNAL(sigNotifyAboutNativeFullscreenDidEnter()),
-                    this, SLOT(sltHandleNativeFullscreenDidEnter()));
+                    this, SLOT(sltHandleNativeFullscreenDidEnter()),
+                    Qt::QueuedConnection);
             connect(pMachineWindow, SIGNAL(sigNotifyAboutNativeFullscreenWillExit()),
-                    this, SLOT(sltHandleNativeFullscreenWillExit()));
+                    this, SLOT(sltHandleNativeFullscreenWillExit()),
+                    Qt::QueuedConnection);
             connect(pMachineWindow, SIGNAL(sigNotifyAboutNativeFullscreenDidExit()),
-                    this, SLOT(sltHandleNativeFullscreenDidExit()));
+                    this, SLOT(sltHandleNativeFullscreenDidExit()),
+                    Qt::QueuedConnection);
+            connect(pMachineWindow, SIGNAL(sigNotifyAboutNativeFullscreenFailToEnter()),
+                    this, SLOT(sltHandleNativeFullscreenFailToEnter()),
+                    Qt::QueuedConnection);
         }
-        /* Revalidate 'fullscreen' windows: */
-        revalidateFullscreenWindows();
+        /* Revalidate native fullscreen: */
+        revalidateNativeFullScreen();
     }
 #endif /* Q_WS_MAC */
 
@@ -471,6 +585,14 @@ void UIMachineLogicFullscreen::cleanupMachineWindows()
         return;
 
 #ifdef Q_WS_MAC
+    /* For ML and next: */
+    if (vboxGlobal().osRelease() > MacOSXRelease_Lion)
+    {
+        /* Unregister from native notifications: */
+        UICocoaApplication::instance()->unregisterFromNotificationOfWorkspace("NSWorkspaceDidActivateApplicationNotification", this);
+        UICocoaApplication::instance()->unregisterFromNotificationOfWorkspace("NSWorkspaceActiveSpaceDidChangeNotification", this);
+    }
+
     /* For Lion and previous fade to black: */
     if (vboxGlobal().osRelease() <= MacOSXRelease_Lion)
         fadeToBlack();
@@ -567,6 +689,7 @@ void UIMachineLogicFullscreen::fadeToBlack()
         return;
 
     /* Acquire fade-token: */
+    LogRel(("UIMachineLogicFullscreen::fadeToBlack\n"));
     CGAcquireDisplayFadeReservation(kCGMaxDisplayReservationInterval, &m_fadeToken);
     CGDisplayFade(m_fadeToken, 0.3, kCGDisplayBlendNormal, kCGDisplayBlendSolidColor, 0.0, 0.0, 0.0, true);
 }
@@ -578,31 +701,59 @@ void UIMachineLogicFullscreen::fadeToNormal()
         return;
 
     /* Release fade-token: */
+    LogRel(("UIMachineLogicFullscreen::fadeToNormal\n"));
     CGDisplayFade(m_fadeToken, 0.5, kCGDisplayBlendSolidColor, kCGDisplayBlendNormal, 0.0, 0.0, 0.0, false);
     CGReleaseDisplayFadeReservation(m_fadeToken);
     m_fadeToken = kCGDisplayFadeReservationInvalidToken;
 }
 
-void UIMachineLogicFullscreen::revalidateFullscreenWindow(UIMachineWindow *pMachineWindow)
+void UIMachineLogicFullscreen::revalidateNativeFullScreen(UIMachineWindow *pMachineWindow)
 {
-    LogRel(("UIMachineLogicFullscreen::revalidateFullscreenWindow #%d begin.\n", pMachineWindow->screenId()));
+    /* Make sure that is full-screen machine-window: */
+    UIMachineWindowFullscreen *pMachineWindowFullscreen = qobject_cast<UIMachineWindowFullscreen*>(pMachineWindow);
+    AssertPtrReturnVoid(pMachineWindowFullscreen);
 
     /* Make sure window is not already invalidated: */
     if (m_invalidFullscreenMachineWindows.contains(pMachineWindow))
         return;
 
     /* Ignore window if it is in 'fullscreen transition': */
-    if (qobject_cast<UIMachineWindowFullscreen*>(pMachineWindow)->isInFullscreenTransition())
+    if (pMachineWindowFullscreen->isInFullscreenTransition())
         return;
 
     /* Get screen ID: */
-    ulong uScreenID = pMachineWindow->screenId();
+    const ulong uScreenID = pMachineWindow->screenId();
+    LogRel(("UIMachineLogicFullscreen::revalidateNativeFullScreen: For machine-window #%d.\n",
+            (int)uScreenID));
 
     /* Validate window which can't be fullscreen: */
-    if (uScreenID != 0 && !darwinScreensHaveSeparateSpaces())
+    if (uScreenID != 0 && !screensHaveSeparateSpaces())
     {
-        /* Make sure window have proper geometry: */
-        pMachineWindow->showInNecessaryMode();
+        /* We are hiding transient window if:
+         * 1. primary window is not on active user-space
+         * 2. there is no fullscreen window or it's invalidated. */
+        if (   !darwinIsOnActiveSpace(machineWindows().first())
+            || m_fullscreenMachineWindows.isEmpty() || !m_invalidFullscreenMachineWindows.isEmpty())
+        {
+            LogRel(("UIMachineLogicFullscreen::revalidateNativeFullScreen: "
+                    "Ask transient machine-window #%d to hide.\n", (int)uScreenID));
+
+            /* Make sure mini-toolbar hidden: */
+            pMachineWindowFullscreen->setMiniToolbarVisible(false);
+            /* Make sure window hidden: */
+            pMachineWindow->hide();
+        }
+        /* If there is valid fullscreen window: */
+        else
+        {
+            LogRel(("UIMachineLogicFullscreen::revalidateNativeFullScreen: "
+                    "Ask transient machine-window #%d to show/normalize.\n", (int)uScreenID));
+
+            /* Make sure window have proper geometry and shown: */
+            pMachineWindow->showInNecessaryMode();
+            /* Make sure mini-toolbar shown: */
+            pMachineWindowFullscreen->setMiniToolbarVisible(true);
+        }
     }
     /* Validate window which can be fullscreen: */
     else
@@ -616,8 +767,8 @@ void UIMachineLogicFullscreen::revalidateFullscreenWindow(UIMachineWindow *pMach
             if (   uisession()->isScreenVisible(uScreenID)
                 && hasHostScreenForGuestScreen(uScreenID))
             {
-                LogRel(("UIMachineLogicFullscreen::revalidateFullscreenWindow: "
-                        "Ask machine-window #%d to enter fullscreen.\n", (int)uScreenID));
+                LogRel(("UIMachineLogicFullscreen::revalidateNativeFullScreen: "
+                        "Ask machine-window #%d to enter native fullscreen.\n", (int)uScreenID));
 
                 /* Fade to black: */
                 fadeToBlack();
@@ -625,6 +776,8 @@ void UIMachineLogicFullscreen::revalidateFullscreenWindow(UIMachineWindow *pMach
                 /* Update 'presentation mode': */
                 setPresentationModeEnabled(true);
 
+                /* Make sure mini-toolbar hidden: */
+                pMachineWindowFullscreen->setMiniToolbarVisible(false);
                 /* Make sure window have proper geometry and shown: */
                 pMachineWindow->showInNecessaryMode();
 
@@ -635,8 +788,13 @@ void UIMachineLogicFullscreen::revalidateFullscreenWindow(UIMachineWindow *pMach
              * is shown while shouldn't: */
             else if (pMachineWindow->isVisible())
             {
-                /* Else make sure that window is hidden: */
-                pMachineWindow->showInNecessaryMode();
+                LogRel(("UIMachineLogicFullscreen::revalidateNativeFullScreen: "
+                        "Ask machine-window #%d to hide.\n", (int)uScreenID));
+
+                /* Make sure mini-toolbar hidden: */
+                pMachineWindowFullscreen->setMiniToolbarVisible(false);
+                /* Make sure window hidden: */
+                pMachineWindow->hide();
 
                 /* Fade to normal: */
                 fadeToNormal();
@@ -654,15 +812,13 @@ void UIMachineLogicFullscreen::revalidateFullscreenWindow(UIMachineWindow *pMach
             /* If that window
              * 1. shouldn't really be shown or
              * 2. isn't mapped to some host-screen or
-             * 3. should be located on another host-screen than currently or
-             * 4. have another frame-buffer size than actually should. */
+             * 3. should be located on another host-screen than currently. */
             if (   !uisession()->isScreenVisible(uScreenID)
                 || !hasHostScreenForGuestScreen(uScreenID)
-                || iWantedHostScreenIndex != iCurrentHostScreenIndex
-                || frameBufferSize != screenSize)
+                || iWantedHostScreenIndex != iCurrentHostScreenIndex)
             {
-                LogRel(("UIMachineLogicFullscreen::revalidateFullscreenWindow: "
-                        "Ask machine-window #%d to exit fullscreen.\n", (int)uScreenID));
+                LogRel(("UIMachineLogicFullscreen::revalidateNativeFullScreen: "
+                        "Ask machine-window #%d to exit native fullscreen.\n", (int)uScreenID));
 
                 /* Fade to black: */
                 fadeToBlack();
@@ -670,20 +826,110 @@ void UIMachineLogicFullscreen::revalidateFullscreenWindow(UIMachineWindow *pMach
                 /* Mark window as invalidated: */
                 m_invalidFullscreenMachineWindows << pMachineWindow;
 
+                /* Make sure mini-toolbar hidden: */
+                pMachineWindowFullscreen->setMiniToolbarVisible(false);
                 /* Ask window to exit 'fullscreen' mode: */
                 emit sigNotifyAboutNativeFullscreenShouldBeExited(pMachineWindow);
+                return;
             }
+
+            /* If that window
+             * 1. have another frame-buffer size than actually should. */
+            else if (frameBufferSize != screenSize)
+            {
+                LogRel(("UIMachineLogicFullscreen::revalidateNativeFullScreen: "
+                        "Ask machine-window #%d to adjust guest geometry.\n", (int)uScreenID));
+
+                /* Make sure mini-toolbar shown: */
+                pMachineWindowFullscreen->setMiniToolbarVisible(true);
+                /* Just adjust machine-view size if necessary: */
+                pMachineWindow->adjustMachineViewSize();
+                return;
+            }
+
+            /* Make sure mini-toolbar shown: */
+            pMachineWindowFullscreen->setMiniToolbarVisible(true);
         }
     }
-
-    LogRel(("UIMachineLogicFullscreen::revalidateFullscreenWindow #%d end.\n", pMachineWindow->screenId()));
 }
 
-void UIMachineLogicFullscreen::revalidateFullscreenWindows()
+void UIMachineLogicFullscreen::revalidateNativeFullScreen()
 {
     /* Revalidate all fullscreen windows: */
     foreach (UIMachineWindow *pMachineWindow, machineWindows())
-        revalidateFullscreenWindow(pMachineWindow);
+        revalidateNativeFullScreen(pMachineWindow);
+}
+
+/* static */
+void UIMachineLogicFullscreen::nativeHandlerForApplicationActivation(QObject *pObject, const QMap<QString, QString> &userInfo)
+{
+    /* Handle arrived notification: */
+    UIMachineLogicFullscreen *pLogic = qobject_cast<UIMachineLogicFullscreen*>(pObject);
+    AssertPtrReturnVoid(pLogic);
+    {
+        /* Redirect arrived notification: */
+        pLogic->nativeHandlerForApplicationActivation(userInfo);
+    }
+}
+
+void UIMachineLogicFullscreen::nativeHandlerForApplicationActivation(const QMap<QString, QString> &userInfo)
+{
+    /* Make sure we have BundleIdentifier key: */
+    AssertReturnVoid(userInfo.contains("BundleIdentifier"));
+    /* Skip other applications: */
+    QStringList ourBundleIdentifiers;
+    ourBundleIdentifiers << "org.virtualbox.app.VirtualBox";
+    ourBundleIdentifiers << "org.virtualbox.app.VirtualBoxVM";
+    ourBundleIdentifiers << "com.citrix.DesktopPlayerVM";
+    if (!ourBundleIdentifiers.contains(userInfo.value("BundleIdentifier")))
+        return;
+
+    /* Skip if 'screen have separate spaces': */
+    if (screensHaveSeparateSpaces())
+        return;
+
+    /* Skip if there is another than needed user-space is active: */
+    if (!darwinIsOnActiveSpace(machineWindows().first()))
+        return;
+
+    LogRel(("UIMachineLogicFullscreen::nativeHandlerForApplicationActivation: "
+            "Full-screen application activated.\n"));
+
+    /* Revalidate full-screen mode for transient machine-window(s): */
+    foreach (UIMachineWindow *pMachineWindow, machineWindows())
+        if (pMachineWindow->screenId() > 0)
+            revalidateNativeFullScreen(pMachineWindow);
+}
+
+/* static */
+void UIMachineLogicFullscreen::nativeHandlerForActiveSpaceChange(QObject *pObject, const QMap<QString, QString> &userInfo)
+{
+    /* Handle arrived notification: */
+    UIMachineLogicFullscreen *pLogic = qobject_cast<UIMachineLogicFullscreen*>(pObject);
+    AssertPtrReturnVoid(pLogic);
+    {
+        /* Redirect arrived notification: */
+        pLogic->nativeHandlerForActiveSpaceChange(userInfo);
+    }
+}
+
+void UIMachineLogicFullscreen::nativeHandlerForActiveSpaceChange(const QMap<QString, QString>&)
+{
+    /* Skip if 'screen have separate spaces': */
+    if (screensHaveSeparateSpaces())
+        return;
+
+    /* Skip if there is another than needed user-space is active: */
+    if (!darwinIsOnActiveSpace(machineWindows().first()))
+        return;
+
+    LogRel(("UIMachineLogicFullscreen::nativeHandlerForActiveSpaceChange: "
+            "Full-screen user-space activated.\n"));
+
+    /* Revalidate full-screen mode for transient machine-window(s): */
+    foreach (UIMachineWindow *pMachineWindow, machineWindows())
+        if (pMachineWindow->screenId() > 0)
+            revalidateNativeFullScreen(pMachineWindow);
 }
 #endif /* Q_WS_MAC */
 
