@@ -48,10 +48,8 @@ UIMachineViewNormal::UIMachineViewNormal(  UIMachineWindow *pMachineWindow
                     )
     , m_bIsGuestAutoresizeEnabled(gActionPool->action(UIActionIndexRuntime_Toggle_GuestAutoresize)->isChecked())
 {
-    /* Resend the last resize hint if there was a fullscreen or
-     * seamless transition previously. If we were not in graphical
-     * mode initially after the transition this happens when we switch. */
-    maybeResendResizeHint();
+    /* Resend the last resize hint if necessary: */
+    maybeResendSizeHint();
 }
 
 UIMachineViewNormal::~UIMachineViewNormal()
@@ -65,7 +63,7 @@ UIMachineViewNormal::~UIMachineViewNormal()
 
 void UIMachineViewNormal::sltAdditionsStateChanged()
 {
-    maybeAdjustGuestScreenSize();
+    adjustGuestScreenSize();
 }
 
 bool UIMachineViewNormal::eventFilter(QObject *pWatched, QEvent *pEvent)
@@ -144,8 +142,9 @@ void UIMachineViewNormal::prepareConsoleConnections()
 
 void UIMachineViewNormal::saveMachineViewSettings()
 {
-    /* Store guest size in case we are switching to fullscreen: */
-    storeGuestSizeHint(QSize(frameBuffer()->width(), frameBuffer()->height()));
+    /* If guest screen-still visible => store it's size-hint: */
+    if (uisession()->isScreenVisible(screenId()))
+        storeGuestSizeHint(QSize(frameBuffer()->width(), frameBuffer()->height()));
 }
 
 void UIMachineViewNormal::setGuestAutoresizeEnabled(bool fEnabled)
@@ -159,14 +158,7 @@ void UIMachineViewNormal::setGuestAutoresizeEnabled(bool fEnabled)
     }
 }
 
-/**
- * Resends guest size-hint if necessary.
- * If the last guest size hint was sent to switch to
- * fullscreen or seamless mode then send one to restore the old view size.
- * @note This method also does some hacks to suppress intermediary resizes
- *       to the old fullscreen size.
- */
-void UIMachineViewNormal::maybeResendResizeHint()
+void UIMachineViewNormal::maybeResendSizeHint()
 {
     if (m_bIsGuestAutoresizeEnabled && uisession()->isGuestSupportsGraphics())
     {
@@ -179,22 +171,24 @@ void UIMachineViewNormal::maybeResendResizeHint()
         QString strHintSent = machine.GetExtraData(strKey);
         if (!strHintSent.isEmpty())
         {
-            QSize hint = guestSizeHint();
+            const QSize sizeHint = guestSizeHint();
+            LogRel(("UIMachineViewNormal::maybeResendSizeHint: "
+                    "Restoring guest size-hint for screen %d to %dx%d\n",
+                    (int)screenId(), sizeHint.width(), sizeHint.height()));
             /* Temporarily restrict the size to prevent a brief resize to the
              * framebuffer dimensions (see @a UIMachineView::sizeHint()) before
              * the following resize() is acted upon. */
-            setMaximumSize(hint);
-            m_sizeHintOverride = hint;
-            sltPerformGuestResize(hint);
+            setMaximumSize(sizeHint);
+            m_sizeHintOverride = sizeHint;
+            sltPerformGuestResize(sizeHint);
         }
     }
 }
 
-/** Adjusts guest screen size to correspond current machine-window size. */
-void UIMachineViewNormal::maybeAdjustGuestScreenSize()
+void UIMachineViewNormal::adjustGuestScreenSize()
 {
-    /* Check if we should adjust guest to new size: */
-    QSize centralWidgetSize = machineWindow()->centralWidget()->size();
+    /* Check if we should adjust guest-screen to new size: */
+    const QSize centralWidgetSize = machineWindow()->centralWidget()->size();
     if ((int)frameBuffer()->width() != centralWidgetSize.width() ||
         (int)frameBuffer()->height() != centralWidgetSize.height())
         if (m_bIsGuestAutoresizeEnabled && uisession()->isGuestSupportsGraphics())
