@@ -76,11 +76,13 @@ bool UIMachineLogicSeamless::checkAvailability()
     return true;
 }
 
-/** Adjusts guest screen count/size for the machine-logic we have. */
-void UIMachineLogicSeamless::maybeAdjustGuestScreenSize()
+void UIMachineLogicSeamless::adjustMachineWindowsGeometry()
 {
-    /* We should rebuild screen-layout: */
+    LogRel(("UIMachineLogicSeamless::adjustMachineWindowsGeometry\n"));
+
+    /* Rebuild multi-screen layout: */
     m_pScreenLayout->rebuild();
+
     /* Make sure all machine-window(s) have proper geometry: */
     foreach (UIMachineWindow *pMachineWindow, machineWindows())
         pMachineWindow->showInNecessaryMode();
@@ -117,7 +119,8 @@ void UIMachineLogicSeamless::sltCheckForRequestedVisualStateType()
     /* If 'seamless' visual-state type is no more supported: */
     if (!uisession()->isGuestSupportsSeamless())
     {
-        LogRel(("UIMachineLogicSeamless: Leaving 'seamless' as it is no more supported...\n"));
+        LogRel(("UIMachineLogicSeamless::sltCheckForRequestedVisualStateType: "
+                "Leaving 'seamless' as it is no more supported...\n"));
         uisession()->setRequestedVisualState(UIVisualStateType_Seamless);
         uisession()->changeVisualState(UIVisualStateType_Normal);
     }
@@ -131,22 +134,21 @@ void UIMachineLogicSeamless::sltMachineStateChanged()
     /* If machine-state changed from 'paused' to 'running': */
     if (uisession()->isRunning() && uisession()->wasPaused())
     {
-        LogRelFlow(("UIMachineLogicSeamless: "
-                    "Machine-state changed from 'paused' to 'running': "
-                    "Updating screen-layout...\n"));
+        LogRel(("UIMachineLogicSeamless::sltMachineStateChanged:"
+                "Machine-state changed from 'paused' to 'running': "
+                "Adjust machine-window geometry...\n"));
 
         /* Make sure further code will be called just once: */
         uisession()->forgetPreviousMachineState();
-        /* We should rebuild screen-layout: */
-        m_pScreenLayout->rebuild();
-        /* Make sure all machine-window(s) have proper geometry: */
-        foreach (UIMachineWindow *pMachineWindow, machineWindows())
-            pMachineWindow->showInNecessaryMode();
+        /* Adjust machine-window geometry if necessary: */
+        adjustMachineWindowsGeometry();
     }
 }
 
 void UIMachineLogicSeamless::sltScreenLayoutChanged()
 {
+    LogRel(("UIMachineLogicSeamless::sltScreenLayoutChanged: Multi-screen layout changed.\n"));
+
     /* Make sure all machine-window(s) have proper geometry: */
     foreach (UIMachineWindow *pMachineWindow, machineWindows())
         pMachineWindow->showInNecessaryMode();
@@ -154,26 +156,24 @@ void UIMachineLogicSeamless::sltScreenLayoutChanged()
 
 void UIMachineLogicSeamless::sltGuestMonitorChange(KGuestMonitorChangedEventType changeType, ulong uScreenId, QRect screenGeo)
 {
-    LogRelFlow(("UIMachineLogicSeamless: Guest-screen count changed.\n"));
+    LogRel(("UIMachineLogicSeamless: Guest-screen count changed.\n"));
 
-    /* Update multi-screen layout before any window update: */
-    if (changeType == KGuestMonitorChangedEventType_Enabled ||
-        changeType == KGuestMonitorChangedEventType_Disabled)
-        m_pScreenLayout->rebuild();
+    /* Rebuild multi-screen layout: */
+    m_pScreenLayout->rebuild();
 
     /* Call to base-class: */
     UIMachineLogic::sltGuestMonitorChange(changeType, uScreenId, screenGeo);
 }
 
-void UIMachineLogicSeamless::sltHostScreenCountChanged()
+void UIMachineLogicSeamless::sltHostScreenCountChange()
 {
-    LogRelFlow(("UIMachineLogicSeamless: Host-screen count changed.\n"));
+    LogRel(("UIMachineLogicSeamless: Host-screen count changed.\n"));
 
-    /* Update multi-screen layout before any window update: */
+    /* Rebuild multi-screen layout: */
     m_pScreenLayout->rebuild();
 
     /* Call to base-class: */
-    UIMachineLogic::sltHostScreenCountChanged();
+    UIMachineLogic::sltHostScreenCountChange();
 }
 
 void UIMachineLogicSeamless::prepareActionGroups()
