@@ -71,7 +71,7 @@ UIMachineViewSeamless::~UIMachineViewSeamless()
 
 void UIMachineViewSeamless::sltAdditionsStateChanged()
 {
-    maybeAdjustGuestScreenSize();
+    adjustGuestScreenSize();
 }
 
 void UIMachineViewSeamless::sltHandleSetVisibleRegion(QRegion region)
@@ -153,10 +153,9 @@ void UIMachineViewSeamless::cleanupSeamless()
         session().GetConsole().GetDisplay().SetSeamlessMode(false);
 }
 
-/** Adjusts guest screen size to correspond current <i>working area</i> size. */
-void UIMachineViewSeamless::maybeAdjustGuestScreenSize()
+void UIMachineViewSeamless::adjustGuestScreenSize()
 {
-    /* Check if we should adjust guest to new size: */
+    /* Check if we should adjust guest-screen to new size: */
     if (frameBuffer()->isAutoEnabled() ||
         (int)frameBuffer()->width() != workingArea().size().width() ||
         (int)frameBuffer()->height() != workingArea().size().height())
@@ -173,7 +172,7 @@ QRect UIMachineViewSeamless::workingArea() const
     /* Get corresponding screen: */
     int iScreen = static_cast<UIMachineLogicSeamless*>(machineLogic())->hostScreenForGuestScreen(screenId());
     /* Return available geometry for that screen: */
-    return vboxGlobal().availableGeometry(iScreen);
+    return QApplication::desktop()->availableGeometry(iScreen);
 }
 
 QSize UIMachineViewSeamless::calculateMaxGuestSize() const
