@@ -17,15 +17,43 @@
 extern "C" {
 #endif
 
+#define CR_CMDVBVA_VERSION              1
+
+#pragma pack(1)
+typedef struct CR_CAPS_INFO
+{
+    uint32_t u32Caps;
+    uint32_t u32CmdVbvaVersion;
+} CR_CAPS_INFO;
+#pragma pack()
+
+
 /*For now guest is allowed to connect host opengl service if protocol version matches exactly*/
 /*Note: that after any change to this file, or glapi_parser\apispec.txt version should be changed*/
 #define CR_PROTOCOL_VERSION_MAJOR 9
 #define CR_PROTOCOL_VERSION_MINOR 1
 
 /* new TexPresent mechanism is available */
-#define CR_VBOX_CAP_TEX_PRESENT    0x00000001
+#define CR_VBOX_CAP_TEX_PRESENT         0x00000001
 /* vbva command submission mechanism supported */
-#define CR_VBOX_CAP_CMDVBVA        0x00000002
+#define CR_VBOX_CAP_CMDVBVA             0x00000002
+/* host supports Command Blocks, i.e. CR_CMDBLOCKBEGIN_OPCODE and CR_CMDBLOCKEND_OPCODE opcodes.
+ * Command Block can be used by guest to prevent clients from blocking each other.
+ * The Command Block allows multiple command buffers to be processed with one run.
+ * Command Block commands have to obey to the following rules:
+ * CR_CMDBLOCKBEGIN_OPCODE - must be the first command in the command buffer, specifying the command block start
+ * CR_CMDBLOCKEND_OPCODE - must be the last command in the command buffer, specifying the command block end
+ * If not placed accordingly, CR_CMDBLOCK** commands are ignored.
+ * Server copies the command block buffer commands to its internal storage
+ * and processes them with one run when the command block end is signalled
+ */
+#define CR_VBOX_CAP_CMDBLOCKS            0x00000004
+/* GetAttribsLocations support */
+#define CR_VBOX_CAP_GETATTRIBSLOCATIONS  0x00000008
+/* flush command blocks for execution  */
+#define CR_VBOX_CAP_CMDBLOCKS_FLUSH      0x00000010
+
+#define CR_VBOX_CAPS_ALL                 0x0000001f
 
 
 #define CR_PRESENT_SCREEN_MASK 0xffff
@@ -102,7 +130,7 @@ typedef struct CRVBOXHGSMI_CMDDATA {
     {
         struct VBOXVDMACMD_CHROMIUM_CMD *pHgsmiCmd;
         struct VBOXCMDVBVA_CRCMD_CMD *pVbvaCmd;
-        void *pvCmd;
+        const void *pvCmd;
     };
     int          *pCmdRc;
     char         *pWriteback;
