@@ -993,7 +993,7 @@ crNetRecvFlowControl( CRConnection *conn,   CRMessageFlowControl *msg,
     conn->InstantReclaim( conn, (CRMessage *) msg );
 }
 
-
+#ifdef IN_GUEST
 /**
  * Called by the main receive function when we get a CR_MESSAGE_WRITEBACK
  * message.  Writeback is used to implement glGet*() functions.
@@ -1026,7 +1026,7 @@ crNetRecvReadback( CRMessageReadback *rb, unsigned int len )
     (*writeback)--;
     crMemcpy( dest_ptr, ((char *)rb) + sizeof(*rb), payload_len );
 }
-
+#endif
 
 /**
  * This is used by the SPUs that do packing (such as Pack, Tilesort and
@@ -1104,13 +1104,21 @@ crNetDefaultRecv( CRConnection *conn, CRMessage *msg, unsigned int len )
             }
             break;
         case CR_MESSAGE_READ_PIXELS:
-            crError( "Can't handle read pixels" );
+            WARN(( "Can't handle read pixels" ));
             return;
         case CR_MESSAGE_WRITEBACK:
+#ifdef IN_GUEST
             crNetRecvWriteback( &(pRealMsg->writeback) );
+#else
+            WARN(("CR_MESSAGE_WRITEBACK not expected\n"));
+#endif
             return;
         case CR_MESSAGE_READBACK:
+#ifdef IN_GUEST
             crNetRecvReadback( &(pRealMsg->readback), len );
+#else
+            WARN(("CR_MESSAGE_READBACK not expected\n"));
+#endif
             return;
         case CR_MESSAGE_CRUT:
             /* nothing */
@@ -1128,10 +1136,10 @@ crNetDefaultRecv( CRConnection *conn, CRMessage *msg, unsigned int len )
             {
                 char string[128];
                 crBytesToString( string, sizeof(string), msg, len );
-                crError("crNetDefaultRecv: received a bad message: type=%d buf=[%s]\n"
+                WARN(("crNetDefaultRecv: received a bad message: type=%d buf=[%s]\n"
                                 "Did you add a new message type and forget to tell "
                                 "crNetDefaultRecv() about it?\n",
-                                msg->header.type, string );
+                                msg->header.type, string ));
             }
     }
 
@@ -1267,6 +1275,18 @@ void crNetReadline( CRConnection *conn, void *buf )
     }
 }
 
+#ifdef IN_GUEST
+uint32_t crNetHostCapsGet()
+{
+#ifdef VBOX_WITH_HGCM
+    if ( cr_net.use_hgcm )
+        return crVBoxHGCMHostCapsGet();
+#endif
+    WARN(("HostCaps supportted for HGCM only!"));
+    return 0;
+}
+#endif
+
 /**
  * The big boy -- call this function to see (non-blocking) if there is
  * any pending work.  If there is, the networking layer's "work received"
