@@ -184,6 +184,10 @@ typedef struct {
 #else
     int                  iGuestDrv;
 #endif
+#ifdef IN_GUEST
+    uint32_t             u32HostCaps;
+    bool                 fHostCapsInitialized;
+#endif
 #if defined(VBOX_WITH_CRHGSMI) && defined(IN_GUEST)
     bool bHgsmiOn;
 #endif
@@ -1262,17 +1266,59 @@ static int crVBoxHGCMSetVersion(CRConnection *conn, unsigned int vMajor, unsigne
 
     rc = crVBoxHGCMCall(conn, &parms, sizeof(parms));
 
-    if (RT_FAILURE(rc) || RT_FAILURE(parms.hdr.result))
+    if (RT_SUCCESS(rc))
     {
-        crWarning("Host doesn't accept our version %d.%d. Make sure you have appropriate additions installed!",
-                  parms.vMajor.u.value32, parms.vMinor.u.value32);
+        rc =  parms.hdr.result;
+        if (RT_SUCCESS(rc))
+        {
+            conn->vMajor = CR_PROTOCOL_VERSION_MAJOR;
+            conn->vMinor = CR_PROTOCOL_VERSION_MINOR;
+
+            return VINF_SUCCESS;
+        }
+        else
+            WARN(("Host doesn't accept our version %d.%d. Make sure you have appropriate additions installed!",
+                  parms.vMajor.u.value32, parms.vMinor.u.value32));
+    }
+    else
+        WARN(("crVBoxHGCMCall failed %d", rc));
+
+    return rc;
+}
+
+static int crVBoxHGCMGetHostCapsLegacy(CRConnection *conn, uint32_t *pu32HostCaps)
+{
+    CRVBOXHGCMGETCAPS caps;
+    int rc;
+
+    caps.hdr.result      = VERR_WRONG_ORDER;
+    caps.hdr.u32ClientID = conn->u32ClientID;
+    caps.hdr.u32Function = SHCRGL_GUEST_FN_GET_CAPS_LEGACY;
+    caps.hdr.cParms      = SHCRGL_CPARMS_GET_CAPS_LEGACY;
+
+    caps.Caps.type       = VMMDevHGCMParmType_32bit;
+    caps.Caps.u.value32  = 0;
+
+    rc = crVBoxHGCMCall(conn, &caps, sizeof(caps));
+
+    if (RT_SUCCESS(rc))
+    {
+        rc =  caps.hdr.result;
+        if (RT_SUCCESS(rc))
+        {
+            *pu32HostCaps = caps.Caps.u.value32;
+            return VINF_SUCCESS;
+        }
+        else
+            WARN(("SHCRGL_GUEST_FN_GET_CAPS failed %d", rc));
         return FALSE;
     }
+    else
+        WARN(("crVBoxHGCMCall failed %d", rc));
 
-    conn->vMajor = CR_PROTOCOL_VERSION_MAJOR;
-    conn->vMinor = CR_PROTOCOL_VERSION_MINOR;
+    *pu32HostCaps = 0;
 
-    return TRUE;
+    return rc;
 }
 
 static int crVBoxHGCMSetPID(CRConnection *conn, unsigned long long pid)
@@ -1379,17 +1425,38 @@ static int crVBoxHGCMDoConnect( CRConnection *conn )
             crDebug("HGCM connect was successful: client id =0x%x\n", conn->u32ClientID);
 
             rc = crVBoxHGCMSetVersion(conn, CR_PROTOCOL_VERSION_MAJOR, CR_PROTOCOL_VERSION_MINOR);
-            if (!rc)
+            if (RT_FAILURE(rc))
             {
-                return rc;
+                WARN(("crVBoxHGCMSetVersion failed %d", rc));
+                return FALSE;
             }
 #ifdef RT_OS_WINDOWS
             rc = crVBoxHGCMSetPID(conn, GetCurrentProcessId());
 #else
             rc = crVBoxHGCMSetPID(conn, crGetPID());
 #endif
+            if (RT_FAILURE(rc))
+            {
+                WARN(("crVBoxHGCMSetPID failed %d", rc));
+                return FALSE;
+            }
+
+            if (!g_crvboxhgcm.fHostCapsInitialized)
+            {
+                rc = crVBoxHGCMGetHostCapsLegacy(conn, &g_crvboxhgcm.u32HostCaps);
+                if (RT_FAILURE(rc))
+                {
+                    WARN(("VBoxCrHgsmiCtlConGetHostCaps failed %d", rc));
+                    g_crvboxhgcm.u32HostCaps = 0;
+                }
+
+                /* host may not support it, ignore any failures */
+                g_crvboxhgcm.fHostCapsInitialized = true;
+                rc = VINF_SUCCESS;
+            }
+
             VBOXCRHGSMIPROFILE_FUNC_EPILOGUE();
-            return rc;
+            return RT_SUCCESS(rc);
         }
         else
         {
@@ -1412,7 +1479,7 @@ static int crVBoxHGCMDoConnect( CRConnection *conn )
     }
 
     VBOXCRHGSMIPROFILE_FUNC_EPILOGUE();
-    return TRUE;
+    return FALSE;
 
 #else /*#ifdef IN_GUEST*/
     crError("crVBoxHGCMDoConnect called on host side!");
@@ -1637,7 +1704,6 @@ static void _crVBoxHGSMIFree(CRConnection *conn, void *buf)
     {
         PVBOXUHGSMI_BUFFER pBuf = _crVBoxHGSMIBufFromHdr(hgcm_buffer);
         PCRVBOXHGSMI_CLIENT pClient = (PCRVBOXHGSMI_CLIENT)pBuf->pvUserData;
-        pBuf->pfnUnlock(pBuf);
         _crVBoxHGSMIBufFree(pClient, pBuf);
     }
     else
@@ -1867,6 +1933,10 @@ _crVBoxHGSMIWriteReadExact(CRConnection *conn, PCRVBOXHGSMI_CLIENT pClient, void
         {
             uint32_t cbWriteback = parms->cbWriteback;
             rc = parms->hdr.result;
+#ifdef DEBUG_misha
+            /* catch it here to test the buffer */
+            Assert(RT_SUCCESS(parms->hdr.result) || parms->hdr.result == VERR_BUFFER_OVERFLOW);
+#endif
             _crVBoxHGSMICmdBufferUnlock(pClient);
 #ifdef DEBUG
             parms = NULL;
@@ -2196,9 +2266,31 @@ static int crVBoxHGSMIDoConnect( CRConnection *conn )
 
     pClient = _crVBoxHGSMIClientGet(conn);
     if (pClient)
+    {
         rc = VBoxCrHgsmiCtlConGetClientID(pClient->pHgsmi, &conn->u32ClientID);
+        if (RT_FAILURE(rc))
+        {
+            WARN(("VBoxCrHgsmiCtlConGetClientID failed %d", rc));
+        }
+        if (!g_crvboxhgcm.fHostCapsInitialized)
+        {
+            rc = VBoxCrHgsmiCtlConGetHostCaps(pClient->pHgsmi, &g_crvboxhgcm.u32HostCaps);
+            if (RT_SUCCESS(rc))
+            {
+                g_crvboxhgcm.fHostCapsInitialized = true;
+            }
+            else
+            {
+                WARN(("VBoxCrHgsmiCtlConGetHostCaps failed %d", rc));
+                g_crvboxhgcm.u32HostCaps = 0;
+            }
+        }
+    }
     else
+    {
+        WARN(("_crVBoxHGSMIClientGet failed %d", rc));
         rc = VERR_GENERAL_FAILURE;
+    }
 
     VBOXCRHGSMIPROFILE_FUNC_EPILOGUE();
 
@@ -2305,6 +2397,11 @@ void crVBoxHGCMInit(CRNetReceiveFuncList *rfl, CRNetCloseFuncList *cfl, unsigned
     crInitMutex(&g_crvboxhgcm.recvmutex);
 #endif
     g_crvboxhgcm.bufpool = crBufferPoolInit(16);
+
+#ifdef IN_GUEST
+    g_crvboxhgcm.fHostCapsInitialized = false;
+    g_crvboxhgcm.u32HostCaps = 0;
+#endif
 }
 
 /* Callback function used to free buffer pool entries */
@@ -2441,6 +2538,10 @@ void crVBoxHGCMConnection(CRConnection *conn
     CRASSERT(conn->pHostBuffer);
     conn->cbHostBuffer = 0;
 
+#if !defined(IN_GUEST)
+    RTListInit(&conn->PendingMsgList);
+#endif
+
 #ifdef CHROMIUM_THREADSAFE
     crLockMutex(&g_crvboxhgcm.mutex);
 #endif
@@ -2500,6 +2601,14 @@ void _crVBoxHGCMPerformReceiveMessage(CRConnection *conn)
     }
 }
 
+#ifdef IN_GUEST
+uint32_t crVBoxHGCMHostCapsGet()
+{
+    Assert(g_crvboxhgcm.fHostCapsInitialized);
+    return g_crvboxhgcm.u32HostCaps;
+}
+#endif
+
 int crVBoxHGCMRecv(
 #if defined(VBOX_WITH_CRHGSMI) && defined(IN_GUEST)
         CRConnection *conn
@@ -2516,7 +2625,6 @@ int crVBoxHGCMRecv(
 
 #ifdef IN_GUEST
 # if defined(VBOX_WITH_CRHGSMI) && defined(IN_GUEST)
-    CRASSERT(!g_crvboxhgcm.bHgsmiOn == !conn);
     if (conn && g_crvboxhgcm.bHgsmiOn)
     {
         _crVBoxHGCMPerformPollHost(conn);
