@@ -47,19 +47,11 @@
 
 #define VBOX_HARDENED_STUB_WITHOUT_IMPORTS
 #ifdef VBOX_HARDENED_STUB_WITHOUT_IMPORTS
-# if 0
-#  define SUPHNTIMP_ERROR(a_fReportErrors, a_id, a_szWhere, a_enmOp, a_rc, ...) \
+# define SUPHNTIMP_ERROR(a_fReportErrors, a_id, a_szWhere, a_enmOp, a_rc, ...) \
     do { \
         if (a_fReportErrors) supR3HardenedFatalMsg(a_szWhere, a_enmOp, a_rc, __VA_ARGS__); \
         else { static const char s_szWhere[] = a_szWhere; *(char *)(uintptr_t)(a_id) += 1; __debugbreak(); } \
     } while (0)
-# else
-#  define SUPHNTIMP_ERROR(a_fReportErrors, a_id, a_FatalErrorMsgArgs) \
-    do { \
-        if (a_fReportErrors) supR3HardenedFatalMsg a_FatalErrorMsgArgs; \
-        else { *(char *)(uintptr_t)(a_id) += 1; __debugbreak(); } \
-    } while (0)
-# endif
 #else
 # define SUPHNTIMP_ERROR(a_fReportErrors, a_id, a_szWhere, a_enmOp, a_rc, ...) \
     supR3HardenedFatalMsg(a_szWhere, a_enmOp, a_rc, __VA_ARGS__)
@@ -99,9 +91,9 @@ typedef struct SUPHNTIMPSYSCALL
     uint32_t               *puApiNo;
     /** Assembly system call routine, type 1.  */
     PFNRT                   pfnType1;
+#ifdef RT_ARCH_X86
     /** Assembly system call routine, type 2.  */
     PFNRT                   pfnType2;
-#ifdef RT_ARCH_X86
     /** The parameter size in bytes for a standard call. */
     uint32_t                cbParams;
 #endif
@@ -222,7 +214,7 @@ static const SUPHNTIMPFUNC g_aSupNtImpKernel32Functions[] =
 # define SUPHARNT_IMPORT_STDCALL(a_Name, a_cbParamsX86) \
     { NULL, NULL },
 # define SUPHARNT_IMPORT_SYSCALL(a_Name, a_cbParamsX86) \
-    { &RT_CONCAT(g_uApiNo, a_Name), &RT_CONCAT(a_Name, _SyscallType1), &RT_CONCAT(a_Name, _SyscallType2) },
+    { &RT_CONCAT(g_uApiNo, a_Name), &RT_CONCAT(a_Name, _SyscallType1) },
 #elif defined(RT_ARCH_X86)
 # define SUPHARNT_IMPORT_STDCALL(a_Name, a_cbParamsX86) \
     { NULL, NULL, NULL, 0 },
@@ -278,13 +270,13 @@ static void supR3HardenedFindOrLoadModule(PSUPHNTIMPDLL pDll)
     if (!pDll->cImports)
         pDll->pbImageBase = NULL; /* optional */
     else
-        SUPHNTIMP_ERROR(false, 1, ("supR3HardenedFindOrLoadModule", kSupInitOp_Misc, VERR_MODULE_NOT_FOUND,
-                        "Failed to locate %ls", pDll->pwszName));
+        SUPHNTIMP_ERROR(false, 1, "supR3HardenedFindOrLoadModule", kSupInitOp_Misc, VERR_MODULE_NOT_FOUND,
+                        "Failed to locate %ls", pDll->pwszName);
 #else
     HMODULE hmod = GetModuleHandleW(pDll->pwszName);
     if (RT_UNLIKELY(!hmod && pDll->cImports))
-        SUPHNTIMP_ERROR(true, 1, ("supR3HardenedWinInitImports", kSupInitOp_Misc, VERR_MODULE_NOT_FOUND,
-                        "Failed to locate %ls", pDll->pwszName));
+        SUPHNTIMP_ERROR(true, 1, "supR3HardenedWinInitImports", kSupInitOp_Misc, VERR_MODULE_NOT_FOUND,
+                        "Failed to locate %ls", pDll->pwszName);
     pDll->pbImageBase = (uint8_t *)hmod;
 #endif
 }
@@ -302,23 +294,23 @@ static void supR3HardenedParseModule(PSUPHNTIMPDLL pDll)
     {
         offNtHdrs = pMzHdr->e_lfanew;
         if (offNtHdrs > _2K)
-            SUPHNTIMP_ERROR(false, 2, ("supR3HardenedParseModule", kSupInitOp_Misc, VERR_MODULE_NOT_FOUND,
-                            "%ls: e_lfanew=%#x, expected a lower value", pDll->pwszName, offNtHdrs));
+            SUPHNTIMP_ERROR(false, 2, "supR3HardenedParseModule", kSupInitOp_Misc, VERR_MODULE_NOT_FOUND,
+                            "%ls: e_lfanew=%#x, expected a lower value", pDll->pwszName, offNtHdrs);
     }
     pDll->pNtHdrs = pNtHdrs = (PIMAGE_NT_HEADERS)&pDll->pbImageBase[offNtHdrs];
 
     if (pNtHdrs->Signature != IMAGE_NT_SIGNATURE)
-        SUPHNTIMP_ERROR(false, 3, ("supR3HardenedParseModule", kSupInitOp_Misc, VERR_INVALID_EXE_SIGNATURE,
-                        "%ls: Invalid PE signature: %#x", pDll->pwszName, pNtHdrs->Signature));
+        SUPHNTIMP_ERROR(false, 3, "supR3HardenedParseModule", kSupInitOp_Misc, VERR_INVALID_EXE_SIGNATURE,
+                        "%ls: Invalid PE signature: %#x", pDll->pwszName, pNtHdrs->Signature);
     if (pNtHdrs->FileHeader.SizeOfOptionalHeader != sizeof(pNtHdrs->OptionalHeader))
-        SUPHNTIMP_ERROR(false, 4, ("supR3HardenedParseModule", kSupInitOp_Misc, VERR_INVALID_EXE_SIGNATURE,
-                        "%ls: Unexpected optional header size: %#x", pDll->pwszName, pNtHdrs->FileHeader.SizeOfOptionalHeader));
+        SUPHNTIMP_ERROR(false, 4, "supR3HardenedParseModule", kSupInitOp_Misc, VERR_INVALID_EXE_SIGNATURE,
+                        "%ls: Unexpected optional header size: %#x", pDll->pwszName, pNtHdrs->FileHeader.SizeOfOptionalHeader);
     if (pNtHdrs->OptionalHeader.Magic != RT_CONCAT3(IMAGE_NT_OPTIONAL_HDR,ARCH_BITS,_MAGIC))
-        SUPHNTIMP_ERROR(false, 5, ("supR3HardenedParseModule", kSupInitOp_Misc, VERR_INVALID_EXE_SIGNATURE,
-                        "%ls: Unexpected optional header magic: %#x", pDll->pwszName, pNtHdrs->OptionalHeader.Magic));
+        SUPHNTIMP_ERROR(false, 5, "supR3HardenedParseModule", kSupInitOp_Misc, VERR_INVALID_EXE_SIGNATURE,
+                        "%ls: Unexpected optional header magic: %#x", pDll->pwszName, pNtHdrs->OptionalHeader.Magic);
     if (pNtHdrs->OptionalHeader.NumberOfRvaAndSizes != IMAGE_NUMBEROF_DIRECTORY_ENTRIES)
-        SUPHNTIMP_ERROR(false, 6, ("supR3HardenedParseModule", kSupInitOp_Misc, VERR_INVALID_EXE_SIGNATURE,
-                        "%ls: Unexpected number of RVA and sizes: %#x", pDll->pwszName, pNtHdrs->OptionalHeader.NumberOfRvaAndSizes));
+        SUPHNTIMP_ERROR(false, 6, "supR3HardenedParseModule", kSupInitOp_Misc, VERR_INVALID_EXE_SIGNATURE,
+                        "%ls: Unexpected number of RVA and sizes: %#x", pDll->pwszName, pNtHdrs->OptionalHeader.NumberOfRvaAndSizes);
 
     pDll->offNtHdrs      = offNtHdrs;
     pDll->offEndSectHdrs = offNtHdrs
@@ -334,8 +326,8 @@ static void supR3HardenedParseModule(PSUPHNTIMPDLL pDll)
         || ExpDir.VirtualAddress < pDll->offEndSectHdrs
         || ExpDir.VirtualAddress >= pNtHdrs->OptionalHeader.SizeOfImage
         || ExpDir.VirtualAddress + ExpDir.Size > pNtHdrs->OptionalHeader.SizeOfImage)
-        SUPHNTIMP_ERROR(false, 7, ("supR3HardenedParseModule", kSupInitOp_Misc, VERR_INVALID_EXE_SIGNATURE,
-                        "%ls: Missing or invalid export directory: %#lx LB %#x", pDll->pwszName, ExpDir.VirtualAddress, ExpDir.Size));
+        SUPHNTIMP_ERROR(false, 7, "supR3HardenedParseModule", kSupInitOp_Misc, VERR_INVALID_EXE_SIGNATURE,
+                        "%ls: Missing or invalid export directory: %#lx LB %#x", pDll->pwszName, ExpDir.VirtualAddress, ExpDir.Size);
     pDll->offExportDir = ExpDir.VirtualAddress;
     pDll->cbExportDir  = ExpDir.Size;
 
@@ -345,31 +337,31 @@ static void supR3HardenedParseModule(PSUPHNTIMPDLL pDll)
         || pExpDir->NumberOfFunctions <  1
         || pExpDir->NumberOfNames     >= _1M
         || pExpDir->NumberOfNames     <  1)
-        SUPHNTIMP_ERROR(false, 8, ("supR3HardenedParseModule", kSupInitOp_Misc, VERR_INVALID_EXE_SIGNATURE,
+        SUPHNTIMP_ERROR(false, 8, "supR3HardenedParseModule", kSupInitOp_Misc, VERR_INVALID_EXE_SIGNATURE,
                         "%ls: NumberOfNames or/and NumberOfFunctions are outside the expected range: nof=%#x non=%#x\n",
-                        pDll->pwszName, pExpDir->NumberOfFunctions, pExpDir->NumberOfNames));
+                        pDll->pwszName, pExpDir->NumberOfFunctions, pExpDir->NumberOfNames);
     pDll->cNamedExports = pExpDir->NumberOfNames;
     pDll->cExports      = RT_MAX(pExpDir->NumberOfNames,  pExpDir->NumberOfFunctions);
 
     if (   pExpDir->AddressOfFunctions < pDll->offEndSectHdrs
         || pExpDir->AddressOfFunctions >= pNtHdrs->OptionalHeader.SizeOfImage
         || pExpDir->AddressOfFunctions + pDll->cExports * sizeof(uint32_t) > pNtHdrs->OptionalHeader.SizeOfImage)
-           SUPHNTIMP_ERROR(false, 9, ("supR3HardenedParseModule", kSupInitOp_Misc, VERR_INVALID_EXE_SIGNATURE,
-                           "%ls: Bad AddressOfFunctions: %#x\n", pDll->pwszName, pExpDir->AddressOfFunctions));
+           SUPHNTIMP_ERROR(false, 9, "supR3HardenedParseModule", kSupInitOp_Misc, VERR_INVALID_EXE_SIGNATURE,
+                           "%ls: Bad AddressOfFunctions: %#x\n", pDll->pwszName, pExpDir->AddressOfFunctions);
     pDll->paoffExports = (uint32_t const *)&pDll->pbImageBase[pExpDir->AddressOfFunctions];
 
     if (   pExpDir->AddressOfNames < pDll->offEndSectHdrs
         || pExpDir->AddressOfNames >= pNtHdrs->OptionalHeader.SizeOfImage
         || pExpDir->AddressOfNames + pExpDir->NumberOfNames * sizeof(uint32_t) > pNtHdrs->OptionalHeader.SizeOfImage)
-           SUPHNTIMP_ERROR(false, 10, ("supR3HardenedParseModule", kSupInitOp_Misc, VERR_INVALID_EXE_SIGNATURE,
-                           "%ls: Bad AddressOfNames: %#x\n", pDll->pwszName, pExpDir->AddressOfNames));
+           SUPHNTIMP_ERROR(false, 10, "supR3HardenedParseModule", kSupInitOp_Misc, VERR_INVALID_EXE_SIGNATURE,
+                           "%ls: Bad AddressOfNames: %#x\n", pDll->pwszName, pExpDir->AddressOfNames);
     pDll->paoffNamedExports = (uint32_t const *)&pDll->pbImageBase[pExpDir->AddressOfNames];
 
     if (   pExpDir->AddressOfNameOrdinals < pDll->offEndSectHdrs
         || pExpDir->AddressOfNameOrdinals >= pNtHdrs->OptionalHeader.SizeOfImage
         || pExpDir->AddressOfNameOrdinals + pExpDir->NumberOfNames * sizeof(uint32_t) > pNtHdrs->OptionalHeader.SizeOfImage)
-           SUPHNTIMP_ERROR(false, 11, ("supR3HardenedParseModule", kSupInitOp_Misc, VERR_INVALID_EXE_SIGNATURE,
-                           "%ls: Bad AddressOfNameOrdinals: %#x\n", pDll->pwszName, pExpDir->AddressOfNameOrdinals));
+           SUPHNTIMP_ERROR(false, 11, "supR3HardenedParseModule", kSupInitOp_Misc, VERR_INVALID_EXE_SIGNATURE,
+                           "%ls: Bad AddressOfNameOrdinals: %#x\n", pDll->pwszName, pExpDir->AddressOfNameOrdinals);
     pDll->pau16NameOrdinals = (uint16_t const *)&pDll->pbImageBase[pExpDir->AddressOfNameOrdinals];
 }
 
@@ -386,8 +378,8 @@ static const char *supR3HardenedResolveImport(PSUPHNTIMPDLL pDll, PCSUPHNTIMPFUN
         uint32_t iCur        = iStart + (iEnd - iStart) / 2;
         uint32_t offExpName  = pDll->paoffNamedExports[iCur];
         if (RT_UNLIKELY(offExpName < pDll->offEndSectHdrs || offExpName >= pDll->cbImage))
-            SUPHNTIMP_ERROR(fReportErrors, 12, ("supR3HardenedResolveImport", kSupInitOp_Misc, VERR_SYMBOL_NOT_FOUND,
-                            "%ls: Bad export name entry: %#x (iCur=%#x)", pDll->pwszName, offExpName, iCur));
+            SUPHNTIMP_ERROR(fReportErrors, 12, "supR3HardenedResolveImport", kSupInitOp_Misc, VERR_SYMBOL_NOT_FOUND,
+                            "%ls: Bad export name entry: %#x (iCur=%#x)", pDll->pwszName, offExpName, iCur);
 
         const char *pszExpName = (const char *)&pDll->pbImageBase[offExpName];
         int iDiff = strcmp(pszExpName, pImport->pszName);
@@ -415,16 +407,16 @@ static const char *supR3HardenedResolveImport(PSUPHNTIMPDLL pDll, PCSUPHNTIMPFUN
                 /* Forwarder. */
                 return (const char *)&pDll->pbImageBase[offExport];
             }
-            SUPHNTIMP_ERROR(fReportErrors, 14, ("supR3HardenedResolveImport", kSupInitOp_Misc, VERR_BAD_EXE_FORMAT,
+            SUPHNTIMP_ERROR(fReportErrors, 14, "supR3HardenedResolveImport", kSupInitOp_Misc, VERR_BAD_EXE_FORMAT,
                             "%ls: Name ordinal for '%s' is out of bounds: %#x (max %#x)",
-                            pDll->pwszName, iExpOrdinal, pDll->cExports));
+                            pDll->pwszName, iExpOrdinal, pDll->cExports);
             return NULL;
         }
     }
 
     if (!pImport->fOptional)
-        SUPHNTIMP_ERROR(fReportErrors, 15, ("supR3HardenedResolveImport", kSupInitOp_Misc, VERR_SYMBOL_NOT_FOUND,
-                        "%ls: Failed to resolve '%s'.", pDll->pwszName, pImport->pszName));
+        SUPHNTIMP_ERROR(fReportErrors, 15, "supR3HardenedResolveImport", kSupInitOp_Misc, VERR_SYMBOL_NOT_FOUND,
+                        "%ls: Failed to resolve '%s'.", pDll->pwszName, pImport->pszName);
     *pImport->ppfnImport = NULL;
     return NULL;
 }
@@ -446,8 +438,8 @@ static void supR3HardenedDirectSyscall(PSUPHNTIMPDLL pDll, PCSUPHNTIMPFUNC pImpo
     int rc = RTLdrGetSymbolEx(pLdrEntry->hLdrMod, pbBits, (uintptr_t)pDll->pbImageBase, UINT32_MAX, pImport->pszName, &uValue);
     if (RT_FAILURE(rc))
     {
-        SUPHNTIMP_ERROR(fReportErrors, 16, ("supR3HardenedDirectSyscall", kSupInitOp_Misc, rc,
-                        "%s: RTLdrGetSymbolEx failed on %s: %Rrc", pDll->pszName, pImport->pszName, rc));
+        SUPHNTIMP_ERROR(fReportErrors, 16, "supR3HardenedDirectSyscall", kSupInitOp_Misc, rc,
+                        "%s: RTLdrGetSymbolEx failed on %s: %Rrc", pDll->pszName, pImport->pszName, rc);
         return;
     }
     uintptr_t offSymbol = (uintptr_t)uValue - (uintptr_t)pDll->pbImageBase;
@@ -457,27 +449,14 @@ static void supR3HardenedDirectSyscall(PSUPHNTIMPDLL pDll, PCSUPHNTIMPFUNC pImpo
      * Parse the code and extract the API call number.
      */
 #ifdef RT_ARCH_AMD64
-    /* Pattern #1: XP64/W2K3-64 thru Windows 10 build 10240.
-            0:000> u ntdll!NtCreateSection
-            ntdll!NtCreateSection:
-            00000000`779f1750 4c8bd1          mov     r10,rcx
-            00000000`779f1753 b847000000      mov     eax,47h
-            00000000`779f1758 0f05            syscall
-            00000000`779f175a c3              ret
-            00000000`779f175b 0f1f440000      nop     dword ptr [rax+rax]
-
-       Pattern #2: Windows 10 build 10525+.
-            0:000> u ntdll_7ffc26300000!NtCreateSection
-            ntdll_7ffc26300000!ZwCreateSection:
-            00007ffc`263943e0 4c8bd1          mov     r10,rcx
-            00007ffc`263943e3 b84a000000      mov     eax,4Ah
-            00007ffc`263943e8 f604250803fe7f01 test    byte ptr [SharedUserData+0x308 (00000000`7ffe0308)],1
-            00007ffc`263943f0 7503            jne     ntdll_7ffc26300000!ZwCreateSection+0x15 (00007ffc`263943f5)
-            00007ffc`263943f2 0f05            syscall
-            00007ffc`263943f4 c3              ret
-            00007ffc`263943f5 cd2e            int     2Eh
-            00007ffc`263943f7 c3              ret
-       */
+    /* Pattern #1: XP64/W2K3-64 thru Windows 8.1
+       0:000> u ntdll!NtCreateSection
+       ntdll!NtCreateSection:
+       00000000`779f1750 4c8bd1          mov     r10,rcx
+       00000000`779f1753 b847000000      mov     eax,47h
+       00000000`779f1758 0f05            syscall
+       00000000`779f175a c3              ret
+       00000000`779f175b 0f1f440000      nop     dword ptr [rax+rax] */
     if (   pbFunction[ 0] == 0x4c /* mov r10, rcx */
         && pbFunction[ 1] == 0x8b
         && pbFunction[ 2] == 0xd1
@@ -485,37 +464,14 @@ static void supR3HardenedDirectSyscall(PSUPHNTIMPDLL pDll, PCSUPHNTIMPFUNC pImpo
         //&& pbFunction[ 4] == 0xZZ
         //&& pbFunction[ 5] == 0xYY
         && pbFunction[ 6] == 0x00
-        && pbFunction[ 7] == 0x00)
+        && pbFunction[ 7] == 0x00
+        && pbFunction[ 8] == 0x0f /* syscall */
+        && pbFunction[ 9] == 0x05
+        && pbFunction[10] == 0xc3 /* ret */ )
     {
-        if (   pbFunction[ 8] == 0x0f /* syscall */
-            && pbFunction[ 9] == 0x05
-            && pbFunction[10] == 0xc3 /* ret */ )
-        {
-            *pSyscall->puApiNo = RT_MAKE_U16(pbFunction[4], pbFunction[5]);
-            *pImport->ppfnImport = pSyscall->pfnType1;
-            return;
-        }
-        if (   pbFunction[ 8] == 0xf6 /* test   byte ptr [SharedUserData+0x308 (00000000`7ffe0308)],1 */
-            && pbFunction[ 9] == 0x04
-            && pbFunction[10] == 0x25
-            && pbFunction[11] == 0x08
-            && pbFunction[12] == 0x03
-            && pbFunction[13] == 0xfe
-            && pbFunction[14] == 0x7f
-            && pbFunction[15] == 0x01
-            && pbFunction[16] == 0x75 /* jnz +3 */
-            && pbFunction[17] == 0x03
-            && pbFunction[18] == 0x0f /* syscall*/
-            && pbFunction[19] == 0x05
-            && pbFunction[20] == 0xc3 /* ret */
-            && pbFunction[21] == 0xcd /* int 2eh */
-            && pbFunction[22] == 0x2e
-            && pbFunction[23] == 0xc3 /* ret */ )
-        {
-            *pSyscall->puApiNo = RT_MAKE_U16(pbFunction[4], pbFunction[5]);
-            *pImport->ppfnImport = pSyscall->pfnType2;
-            return;
-        }
+        *pSyscall->puApiNo = RT_MAKE_U16(pbFunction[4], pbFunction[5]);
+        *pImport->ppfnImport = pSyscall->pfnType1;
+        return;
     }
 #else
     /* Pattern #1: XP thru Windows 7
@@ -587,9 +543,9 @@ static void supR3HardenedDirectSyscall(PSUPHNTIMPDLL pDll, PCSUPHNTIMPFUNC pImpo
      */
     volatile uint8_t abCopy[16];
     memcpy((void *)&abCopy[0], pbFunction, sizeof(abCopy));
-    SUPHNTIMP_ERROR(fReportErrors, 17, ("supR3HardenedWinInitImports", kSupInitOp_Misc, rc,
+    SUPHNTIMP_ERROR(fReportErrors, 17, "supR3HardenedWinInitImports", kSupInitOp_Misc, rc,
                     "%ls: supHardNtLdrCacheOpen failed: '%s': %.16Rhxs",
-                    pDll->pwszName, pImport->pszName, &abCopy[0]));
+                    pDll->pwszName, pImport->pszName, &abCopy[0]);
 }
 
 
@@ -621,12 +577,12 @@ DECLHIDDEN(void) supR3HardenedWinInitSyscalls(bool fReportErrors)
                                                    &g_aSupNtImpDlls[iDll].paSyscalls[i], pLdrEntry, pbBits, fReportErrors);
                 }
                 else
-                    SUPHNTIMP_ERROR(fReportErrors, 20, ("supR3HardenedWinInitImports", kSupInitOp_Misc, rc,
-                                    "%ls: supHardNtLdrCacheEntryGetBits failed: %Rrc '%s'.", g_aSupNtImpDlls[iDll].pwszName, rc));
+                    SUPHNTIMP_ERROR(fReportErrors, 20, "supR3HardenedWinInitImports", kSupInitOp_Misc, rc,
+                                    "%ls: supHardNtLdrCacheEntryGetBits failed: %Rrc '%s'.", g_aSupNtImpDlls[iDll].pwszName, rc);
             }
             else
-                SUPHNTIMP_ERROR(fReportErrors, 21, ("supR3HardenedWinInitImports", kSupInitOp_Misc, rc,
-                                "%ls: supHardNtLdrCacheOpen failed: %Rrc '%s'.", g_aSupNtImpDlls[iDll].pwszName, rc));
+                SUPHNTIMP_ERROR(fReportErrors, 21, "supR3HardenedWinInitImports", kSupInitOp_Misc, rc,
+                                "%ls: supHardNtLdrCacheOpen failed: %Rrc '%s'.", g_aSupNtImpDlls[iDll].pwszName, rc);
         }
 }
 
@@ -663,8 +619,8 @@ DECLHIDDEN(void) supR3HardenedWinGetVeryEarlyImports(uintptr_t uNtDllAddr,
         {
             const char *pszForwarder = supR3HardenedResolveImport(&g_aSupNtImpDlls[0], &aImports[i], false);
             if (pszForwarder)
-                SUPHNTIMP_ERROR(false, 31, ("supR3HardenedWinGetVeryEarlyImports", kSupInitOp_Misc, VERR_MODULE_NOT_FOUND,
-                                "ntdll: Failed to resolve forwarder '%s'.", pszForwarder));
+                SUPHNTIMP_ERROR(false, 31, "supR3HardenedWinGetVeryEarlyImports", kSupInitOp_Misc, VERR_MODULE_NOT_FOUND,
+                                "ntdll: Failed to resolve forwarder '%s'.", pszForwarder);
         }
 
     /*
@@ -691,8 +647,8 @@ DECLHIDDEN(void) supR3HardenedWinInitImportsEarly(uintptr_t uNtDllAddr)
         {
             const char *pszForwarder = supR3HardenedResolveImport(&g_aSupNtImpDlls[0], &g_aSupNtImpDlls[0].paImports[i], false);
             if (pszForwarder)
-                SUPHNTIMP_ERROR(false, 32, ("supR3HardenedWinInitImports", kSupInitOp_Misc, VERR_MODULE_NOT_FOUND,
-                                "ntdll: Failed to resolve forwarder '%s'.", pszForwarder));
+                SUPHNTIMP_ERROR(false, 32, "supR3HardenedWinInitImports", kSupInitOp_Misc, VERR_MODULE_NOT_FOUND,
+                                "ntdll: Failed to resolve forwarder '%s'.", pszForwarder);
         }
         else
             *g_aSupNtImpDlls[0].paImports[i].ppfnImport = g_aSupNtImpDlls[0].paImports[i].pfnEarlyDummy;
@@ -749,8 +705,8 @@ DECLHIDDEN(void) supR3HardenedWinInitImports(void)
                 else if (cchDllName == sizeof("kernelbase") - 1 && RTStrNICmp(pszForwarder, RT_STR_TUPLE("kernelbase")) == 0)
                     supR3HardenedResolveImport(&g_aSupNtImpDlls[1], &Tmp, false);
                 else
-                    SUPHNTIMP_ERROR(false, 18, ("supR3HardenedWinInitImports", kSupInitOp_Misc, VERR_MODULE_NOT_FOUND,
-                                    "%ls: Failed to resolve forwarder '%s'.", g_aSupNtImpDlls[iDll].pwszName, pszForwarder));
+                    SUPHNTIMP_ERROR(false, 18, "supR3HardenedWinInitImports", kSupInitOp_Misc, VERR_MODULE_NOT_FOUND,
+                                    "%ls: Failed to resolve forwarder '%s'.", g_aSupNtImpDlls[iDll].pwszName, pszForwarder);
             }
         }
 
