@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2010-2013 Oracle Corporation
+ * Copyright (C) 2010-2014 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -18,8 +18,15 @@
 /*******************************************************************************
 *   Header Files                                                               *
 *******************************************************************************/
+#if defined(_WIN32_WINNT) && _WIN32_WINNT < 0x0501
+# undef  _WIN32_WINNT
+# define _WIN32_WINNT 0x0501 /* AttachConsole() / FreeConsole(). */
+#endif
+
 #include <Windows.h>
 #include <commctrl.h>
+#include <fcntl.h>
+#include <io.h>
 #include <lmerr.h>
 #include <msiquery.h>
 #include <objbase.h>
@@ -55,6 +62,10 @@
 # include "VBoxStubPublicCert.h"
 #endif
 
+#ifndef TARGET_NT4
+/* Use an own console window if run in verbose mode. */
+# define VBOX_STUB_WITH_OWN_CONSOLE
+#endif
 
 /*******************************************************************************
 *   Defined Constants And Macros                                               *
@@ -89,6 +100,8 @@ typedef STUBCLEANUPREC *PSTUBCLEANUPREC;
 static bool             g_fSilent = false;
 /** List of temporary files. */
 static RTLISTANCHOR     g_TmpFiles;
+/** Verbosity flag. */
+static int              g_iVerbosity = 0;
 
 
 
@@ -801,7 +814,12 @@ int WINAPI WinMain(HINSTANCE  hInstance,
     /* Init IPRT. */
     int vrc = RTR3InitExe(argc, &argv, 0);
     if (RT_FAILURE(vrc))
+    {
+        /* Close the mutex for this application instance. */
+        CloseHandle(hMutexAppRunning);
+        hMutexAppRunning = NULL;
         return RTMsgInitFailure(vrc);
+    }
 
     /*
      * Parse arguments.
@@ -814,11 +832,13 @@ int WINAPI WinMain(HINSTANCE  hInstance,
     bool fEnableSilentCert         = true;
 #endif
     char szExtractPath[RTPATH_MAX] = {0};
-    char szMSIArgs[4096]           = {0};
+    char szMSIArgs[_4K]            = {0};
 
     /* Parameter definitions. */
     static const RTGETOPTDEF s_aOptions[] =
     {
+        /** @todo Replace short parameters with enums since they're not
+         *        used (and not documented to the public). */
         { "--extract",          'x', RTGETOPT_REQ_NOTHING },
         { "-extract",           'x', RTGETOPT_REQ_NOTHING },
         { "/extract",           'x', RTGETOPT_REQ_NOTHING },
@@ -838,6 +858,12 @@ int WINAPI WinMain(HINSTANCE  hInstance,
         { "/path",              'p', RTGETOPT_REQ_STRING  },
         { "--msiparams",        'm', RTGETOPT_REQ_STRING  },
         { "-msiparams",         'm', RTGETOPT_REQ_STRING  },
+        { "--reinstall",        'f', RTGETOPT_REQ_NOTHING },
+        { "-reinstall",         'f', RTGETOPT_REQ_NOTHING },
+        { "/reinstall",         'f', RTGETOPT_REQ_NOTHING },
+        { "--verbose",          'v', RTGETOPT_REQ_NOTHING },
+        { "-verbose",           'v', RTGETOPT_REQ_NOTHING },
+        { "/verbose",           'v', RTGETOPT_REQ_NOTHING },
         { "--version",          'V', RTGETOPT_REQ_NOTHING },
         { "-version",           'V', RTGETOPT_REQ_NOTHING },
         { "/version",           'V', RTGETOPT_REQ_NOTHING },
@@ -848,15 +874,30 @@ int WINAPI WinMain(HINSTANCE  hInstance,
         { "/?",                 'h', RTGETOPT_REQ_NOTHING },
     };
 
+    RTEXITCODE rcExit = RTEXITCODE_SUCCESS;
+
     /* Parse the parameters. */
     int ch;
+    bool fParsingDone = false;
     RTGETOPTUNION ValueUnion;
     RTGETOPTSTATE GetState;
     RTGetOptInit(&GetState, argc, argv, s_aOptions, RT_ELEMENTS(s_aOptions), 1, 0);
-    while ((ch = RTGetOpt(&GetState, &ValueUnion)))
+    while (   (ch = RTGetOpt(&GetState, &ValueUnion))
+           && rcExit == RTEXITCODE_SUCCESS
+           && !fParsingDone)
     {
         switch (ch)
         {
+            case 'f': /* Force re-installation. */
+                if (szMSIArgs[0])
+                    vrc = RTStrCat(szMSIArgs, sizeof(szMSIArgs), " ");
+                if (RT_SUCCESS(vrc))
+                    vrc = RTStrCat(szMSIArgs, sizeof(szMSIArgs),
+                                   "REINSTALLMODE=vomus REINSTALL=ALL");
+                if (RT_FAILURE(vrc))
+                    rcExit = ShowError("MSI parameters are too long.");
+                break;
+
             case 'x':
                 fExtractOnly = true;
                 break;
@@ -870,7 +911,6 @@ int WINAPI WinMain(HINSTANCE  hInstance,
                 fEnableSilentCert = false;
                 break;
 #endif
-
             case 'l':
                 fEnableLogging = true;
                 break;
@@ -878,7 +918,7 @@ int WINAPI WinMain(HINSTANCE  hInstance,
             case 'p':
                 vrc = RTStrCopy(szExtractPath, sizeof(szExtractPath), ValueUnion.psz);
                 if (RT_FAILURE(vrc))
-                    return ShowError("Extraction path is too long.");
+                    rcExit = ShowError("Extraction path is too long.");
                 break;
 
             case 'm':
@@ -887,114 +927,197 @@ int WINAPI WinMain(HINSTANCE  hInstance,
                 if (RT_SUCCESS(vrc))
                     vrc = RTStrCat(szMSIArgs, sizeof(szMSIArgs), ValueUnion.psz);
                 if (RT_FAILURE(vrc))
-                    return ShowError("MSI parameters are too long.");
+                    rcExit = ShowError("MSI parameters are too long.");
                 break;
 
             case 'V':
                 ShowInfo("Version: %d.%d.%d.%d",
-                         VBOX_VERSION_MAJOR, VBOX_VERSION_MINOR, VBOX_VERSION_BUILD, VBOX_SVN_REV);
-                return VINF_SUCCESS;
+                         VBOX_VERSION_MAJOR, VBOX_VERSION_MINOR, VBOX_VERSION_BUILD,
+                         VBOX_SVN_REV);
+                fParsingDone = true;
+                break;
+
+            case 'v':
+                g_iVerbosity++;
+                break;
 
             case 'h':
                 ShowInfo("-- %s v%d.%d.%d.%d --\n"
                          "\n"
                          "Command Line Parameters:\n\n"
                          "--extract                - Extract file contents to temporary directory\n"
-                         "--silent                 - Enables silent mode installation\n"
+                         "--help                   - Print this help and exit\n"
+                         "--logging                - Enables installer logging\n"
+                         "--msiparams <parameters> - Specifies extra parameters for the MSI installers\n"
                          "--no-silent-cert         - Do not install VirtualBox Certificate automatically when --silent option is specified\n"
                          "--path                   - Sets the path of the extraction directory\n"
-                         "--msiparams <parameters> - Specifies extra parameters for the MSI installers\n"
-                         "--logging                - Enables installer logging\n"
-                         "--help                   - Print this help and exit\n"
+                         "--reinstall              - Forces VirtualBox to get re-installed\n"
+                         "--silent                 - Enables silent mode installation\n"
                          "--version                - Print version number and exit\n\n"
                          "Examples:\n"
                          "%s --msiparams INSTALLDIR=C:\\VBox\n"
                          "%s --extract -path C:\\VBox",
                          VBOX_STUB_TITLE, VBOX_VERSION_MAJOR, VBOX_VERSION_MINOR, VBOX_VERSION_BUILD, VBOX_SVN_REV,
                          argv[0], argv[0]);
-                return VINF_SUCCESS;
+                fParsingDone = true;
+                break;
+
+            case VINF_GETOPT_NOT_OPTION:
+                /* Are (optional) MSI parameters specified and this is the last
+                 * parameter? Append everything to the MSI parameter list then. */
+                if (szMSIArgs[0])
+                {
+                    vrc = RTStrCat(szMSIArgs, sizeof(szMSIArgs), " ");
+                    if (RT_SUCCESS(vrc))
+                        vrc = RTStrCat(szMSIArgs, sizeof(szMSIArgs), ValueUnion.psz);
+                    if (RT_FAILURE(vrc))
+                        rcExit = ShowError("MSI parameters are too long.");
+                    continue;
+                }
+                /* Fall through is intentional. */
 
             default:
                 if (g_fSilent)
-                    return RTGetOptPrintError(ch, &ValueUnion);
-                if (ch == VINF_GETOPT_NOT_OPTION || ch == VERR_GETOPT_UNKNOWN_OPTION)
-                    ShowError("Unknown option \"%s\"!\n"
-                              "Please refer to the command line help by specifying \"/?\"\n"
-                              "to get more information.", ValueUnion.psz);
+                    rcExit = RTGetOptPrintError(ch, &ValueUnion);
+                if (ch == VERR_GETOPT_UNKNOWN_OPTION)
+                    rcExit = ShowError("Unknown option \"%s\"\n"
+                                       "Please refer to the command line help by specifying \"/?\"\n"
+                                       "to get more information.", ValueUnion.psz);
                 else
-                    ShowError("Parameter parsing error: %Rrc\n"
-                              "Please refer to the command line help by specifying \"/?\"\n"
-                              "to get more information.", ch);
-                return RTEXITCODE_SYNTAX;
-
+                    rcExit = ShowError("Parameter parsing error: %Rrc\n"
+                                       "Please refer to the command line help by specifying \"/?\"\n"
+                                       "to get more information.", ch);
+                break;
         }
     }
 
-    /*
-     * Determine the extration path if not given by the user, and gather some
-     * other bits we'll be needing later.
-     */
-    if (szExtractPath[0] == '\0')
-    {
-        vrc = RTPathTemp(szExtractPath, sizeof(szExtractPath));
-        if (RT_SUCCESS(vrc))
-            vrc = RTPathAppend(szExtractPath, sizeof(szExtractPath), "VirtualBox");
-        if (RT_FAILURE(vrc))
-            return ShowError("Failed to determin extraction path (%Rrc)", vrc);
+    if (rcExit != RTEXITCODE_SUCCESS)
+        vrc = VERR_PARSE_ERROR;
 
-    }
-    else
+#if defined(_WIN32_WINNT) && _WIN32_WINNT >= 0x0501
+# ifdef VBOX_STUB_WITH_OWN_CONSOLE /* Use an own console window if run in debug mode. */
+    if (   RT_SUCCESS(vrc)
+        && g_iVerbosity)
     {
-        /** @todo should check if there is a .custom subdirectory there or not. */
+        if (!AllocConsole())
+        {
+            DWORD dwErr = GetLastError();
+            ShowError("Unable to allocate console, error = %ld\n",
+                      dwErr);
+
+            /* Close the mutex for this application instance. */
+            CloseHandle(hMutexAppRunning);
+            hMutexAppRunning = NULL;
+            return RTEXITCODE_FAILURE;
+        }
+
+        freopen("CONOUT$", "w", stdout);
+        setvbuf(stdout, NULL, _IONBF, 0);
+
+        freopen("CONOUT$", "w", stderr);
+    }
+# endif /* VBOX_STUB_WITH_OWN_CONSOLE */
+#endif
+
+    if (   RT_SUCCESS(vrc)
+        && g_iVerbosity)
+    {
+        RTPrintf("Silent installation      : %RTbool\n", g_fSilent);
+        RTPrintf("Logging enabled          : %RTbool\n", fEnableLogging);
+#ifdef VBOX_WITH_CODE_SIGNING
+        RTPrintf("Certificate installation : %RTbool\n", fEnableSilentCert);
+#endif
+        RTPrintf("Additional MSI parameters: %s\n",
+                 szMSIArgs[0] ? szMSIArgs : "<None>");
+    }
+
+    if (RT_SUCCESS(vrc))
+    {
+        /*
+         * Determine the extration path if not given by the user, and gather some
+         * other bits we'll be needing later.
+         */
+        if (szExtractPath[0] == '\0')
+        {
+            vrc = RTPathTemp(szExtractPath, sizeof(szExtractPath));
+            if (RT_SUCCESS(vrc))
+                vrc = RTPathAppend(szExtractPath, sizeof(szExtractPath), "VirtualBox");
+            if (RT_FAILURE(vrc))
+                ShowError("Failed to determine extraction path (%Rrc)", vrc);
+
+        }
+        else
+        {
+            /** @todo should check if there is a .custom subdirectory there or not. */
+        }
+        RTPathChangeToDosSlashes(szExtractPath,
+                                 true /* Force conversion. */); /* MSI requirement. */
     }
-    RTPathChangeToDosSlashes(szExtractPath, true /* Force conversion. */); /* MSI requirement. */
 
     /* Read our manifest. */
     PVBOXSTUBPKGHEADER pHeader;
-    vrc = FindData("MANIFEST", (PVOID *)&pHeader, NULL);
-    if (RT_FAILURE(vrc))
-        return ShowError("Internal package error: Manifest not found (%Rrc)", vrc);
-    /** @todo If we could, we should validate the header. Only the magic isn't
-     *        commonly defined, nor the version number... */
-
-    RTListInit(&g_TmpFiles);
-
-    /*
-     * Up to this point, we haven't done anything that requires any cleanup.
-     * From here on, we do everything in function so we can counter clean up.
-     */
-    bool fCreatedExtractDir;
-    RTEXITCODE rcExit = ExtractFiles(pHeader->byCntPkgs, szExtractPath, fExtractOnly, &fCreatedExtractDir);
-    if (rcExit == RTEXITCODE_SUCCESS)
+    if (RT_SUCCESS(vrc))
     {
-        if (fExtractOnly)
-            ShowInfo("Files were extracted to: %s", szExtractPath);
-        else
-        {
-            rcExit = CopyCustomDir(szExtractPath);
-#ifdef VBOX_WITH_CODE_SIGNING
-            if (rcExit == RTEXITCODE_SUCCESS && fEnableSilentCert && g_fSilent)
-                rcExit = InstallCertificate();
-#endif
-            unsigned iPackage = 0;
-            while (iPackage < pHeader->byCntPkgs && rcExit == RTEXITCODE_SUCCESS)
-            {
-                rcExit = ProcessPackage(iPackage, szExtractPath, szMSIArgs, fEnableLogging);
-                iPackage++;
-            }
+        vrc = FindData("MANIFEST", (PVOID *)&pHeader, NULL);
+        if (RT_FAILURE(vrc))
+            rcExit = ShowError("Internal package error: Manifest not found (%Rrc)", vrc);
+    }
+    if (RT_SUCCESS(vrc))
+    {
+        /** @todo If we could, we should validate the header. Only the magic isn't
+         *        commonly defined, nor the version number... */
 
-            /* Don't fail if cleanup fail. At least for now. */
-            CleanUp(pHeader->byCntPkgs, !fEnableLogging && fCreatedExtractDir ? szExtractPath : NULL);
+        RTListInit(&g_TmpFiles);
+
+        /*
+         * Up to this point, we haven't done anything that requires any cleanup.
+         * From here on, we do everything in function so we can counter clean up.
+         */
+        bool fCreatedExtractDir;
+        rcExit = ExtractFiles(pHeader->byCntPkgs, szExtractPath,
+                              fExtractOnly, &fCreatedExtractDir);
+        if (rcExit == RTEXITCODE_SUCCESS)
+        {
+            if (fExtractOnly)
+                ShowInfo("Files were extracted to: %s", szExtractPath);
+            else
+            {
+                rcExit = CopyCustomDir(szExtractPath);
+#ifdef VBOX_WITH_CODE_SIGNING
+                if (rcExit == RTEXITCODE_SUCCESS && fEnableSilentCert && g_fSilent)
+                    rcExit = InstallCertificate();
+#endif
+                unsigned iPackage = 0;
+                while (   iPackage < pHeader->byCntPkgs
+                       && rcExit == RTEXITCODE_SUCCESS)
+                {
+                    rcExit = ProcessPackage(iPackage, szExtractPath,
+                                            szMSIArgs, fEnableLogging);
+                    iPackage++;
+                }
+
+                /* Don't fail if cleanup fail. At least for now. */
+                CleanUp(pHeader->byCntPkgs,
+                           !fEnableLogging
+                        && fCreatedExtractDir ? szExtractPath : NULL);
+            }
+        }
+
+        /* Free any left behind cleanup records (not strictly needed). */
+        PSTUBCLEANUPREC pCur, pNext;
+        RTListForEachSafe(&g_TmpFiles, pCur, pNext, STUBCLEANUPREC, ListEntry)
+        {
+            RTListNodeRemove(&pCur->ListEntry);
+            RTMemFree(pCur);
         }
     }
 
-    /* Free any left behind cleanup records (not strictly needed). */
-    PSTUBCLEANUPREC pCur, pNext;
-    RTListForEachSafe(&g_TmpFiles, pCur, pNext, STUBCLEANUPREC, ListEntry)
-    {
-        RTListNodeRemove(&pCur->ListEntry);
-        RTMemFree(pCur);
-    }
+#if defined(_WIN32_WINNT) && _WIN32_WINNT >= 0x0501
+# ifdef VBOX_STUB_WITH_OWN_CONSOLE
+    if (g_iVerbosity)
+        FreeConsole();
+# endif /* VBOX_STUB_WITH_OWN_CONSOLE */
+#endif
 
     /*
      * Release instance mutex.
