@@ -49,6 +49,7 @@ class ExtPackManager;
 class VMMDevMouseInterface;
 class DisplayMouseInterface;
 
+#include <iprt/memsafer.h>
 #include <VBox/RemoteDesktop/VRDE.h>
 #include <VBox/vmm/pdmdrv.h>
 #ifdef VBOX_WITH_GUEST_PROPS
@@ -306,6 +307,22 @@ public:
 
     EmulatedUSB *getEmulatedUSB(void) { return mEmulatedUSB; }
 
+    /**
+     * Sets the disk encryption keys.
+     *
+     * @returns COM status code.
+     * @þaram   strCfg    The config for the disks.
+     *
+     * @note: One line in the config string contains all required data for one disk.
+     *        The format for one disk is some sort of comma separated value using
+     *        key=value pairs.
+     *        There are two keys defined at the moment:
+     *            - uuid: The uuid of the base image the key is for (with or without)
+     *                    the curly braces.
+     *            - dek: The data encryption key in base64 encoding
+     */
+    HRESULT setDiskEncryptionKeys(const Utf8Str &strCfg);
+
 private:
 
     /**
@@ -316,22 +333,20 @@ private:
     class AutoVMCallerBase
     {
     public:
-        AutoVMCallerBase(Console *aThat) : mThat(aThat), mRC(S_OK)
+        AutoVMCallerBase(Console *aThat) : mThat(aThat), mRC(E_FAIL)
         {
             Assert(aThat);
             mRC = aThat->addVMCaller(taQuiet, taAllowNullVM);
         }
         ~AutoVMCallerBase()
         {
-            if (SUCCEEDED(mRC))
-                mThat->releaseVMCaller();
+            doRelease();
         }
         /** Decreases the number of callers before the instance is destroyed. */
         void releaseCaller()
         {
-            AssertReturnVoid(SUCCEEDED(mRC));
-            mThat->releaseVMCaller();
-            mRC = E_FAIL;
+            Assert(SUCCEEDED(mRC));
+            doRelease();
         }
         /** Restores the number of callers after by #release(). #rc() must be
          *  rechecked to ensure the operation succeeded. */
@@ -346,8 +361,16 @@ private:
         bool isOk() const { return SUCCEEDED(mRC); }
     protected:
         Console *mThat;
-        HRESULT mRC;
+        void doRelease()
+        {
+            if (SUCCEEDED(mRC))
+            {
+                mThat->releaseVMCaller();
+                mRC = E_FAIL;
+            }
+        }
     private:
+        HRESULT mRC; /* Whether the caller was added. */
         DECLARE_CLS_COPY_CTOR_ASSIGN_NOOP(AutoVMCallerBase)
     };
 
@@ -407,27 +430,40 @@ private:
     {
         typedef AutoVMCallerBase<taQuiet, true> Base;
     public:
-        SafeVMPtrBase(Console *aThat) : Base(aThat), mpUVM(NULL)
+        SafeVMPtrBase(Console *aThat) : Base(aThat), mRC(E_FAIL), mpUVM(NULL)
         {
-            if (SUCCEEDED(Base::mRC))
-                Base::mRC = aThat->safeVMPtrRetainer(&mpUVM, taQuiet);
+            if (Base::isOk())
+                mRC = aThat->safeVMPtrRetainer(&mpUVM, taQuiet);
         }
         ~SafeVMPtrBase()
         {
-            if (SUCCEEDED(Base::mRC))
-                release();
+            doRelease();
         }
         /** Direct PUVM access. */
         PUVM rawUVM() const { return mpUVM; }
         /** Release the handles. */
         void release()
         {
-            AssertReturnVoid(SUCCEEDED(Base::mRC));
-            Base::mThat->safeVMPtrReleaser(&mpUVM);
-            Base::releaseCaller();
+            Assert(SUCCEEDED(mRC));
+            doRelease();
         }
 
+        /** The combined result of Console::addVMCaller() and Console::safeVMPtrRetainer */
+        HRESULT rc() const { return Base::isOk()? mRC: Base::rc(); }
+        /** Shortcut to SUCCEEDED(rc()) */
+        bool isOk() const { return SUCCEEDED(mRC) && Base::isOk(); }
+
     private:
+        void doRelease()
+        {
+            if (SUCCEEDED(mRC))
+            {
+                Base::mThat->safeVMPtrReleaser(&mpUVM);
+                mRC = E_FAIL;
+            }
+            Base::doRelease();
+        }
+        HRESULT mRC; /* Whether the VM ptr was retained. */
         PUVM    mpUVM;
         DECLARE_CLS_COPY_CTOR_ASSIGN_NOOP(SafeVMPtrBase)
     };
@@ -497,9 +533,41 @@ public:
         bool m_fAutoMount;
     };
 
+    /**
+     * Class for managing cryptographic keys.
+     * @ŧodo: Replace with a keystore implementation once it is ready.
+     */
+    class SecretKey
+    {
+        public:
+            SecretKey() { }
+
+            SecretKey(void *pvKey, size_t cbKey)
+               : m_cRefs(0),
+                 m_pvKey(pvKey),
+                 m_cbKey(cbKey)
+            { }
+
+            ~SecretKey()
+            {
+                RTMemSaferFree(m_pvKey, m_cbKey);
+                m_cRefs = 0;
+                m_pvKey = NULL;
+                m_cbKey = 0;
+            }
+
+            /** Reference counter of the key. */
+            volatile uint32_t m_cRefs;
+            /** Key material. */
+            void    *m_pvKey;
+            /** Size of the key in bytes. */
+            size_t   m_cbKey;
+    };
+
     typedef std::map<Utf8Str, ComObjPtr<SharedFolder> > SharedFolderMap;
     typedef std::map<Utf8Str, SharedFolderData> SharedFolderDataMap;
     typedef std::map<Utf8Str, ComPtr<IMediumAttachment> > MediumAttachmentMap;
+    typedef std::map<Utf8Str, SecretKey *> SecretKeyMap;
 
 private:
 
@@ -694,6 +762,10 @@ private:
     static DECLCALLBACK(void)   drvStatus_Destruct(PPDMDRVINS pDrvIns);
     static DECLCALLBACK(int)    drvStatus_Construct(PPDMDRVINS pDrvIns, PCFGMNODE pCfg, uint32_t fFlags);
 
+    static DECLCALLBACK(int)    i_pdmIfSecKey_KeyRetain(PPDMISECKEY pInterface, const char *pszId, const uint8_t **ppbKey,
+                                                        size_t *pcbKey);
+    static DECLCALLBACK(int)    i_pdmIfSecKey_KeyRelease(PPDMISECKEY pInterface, const char *pszId);
+
     int mcAudioRefs;
     volatile uint32_t mcVRDPClients;
     uint32_t mu32SingleRDPClientId; /* The id of a connected client in the single connection mode. */
@@ -730,6 +802,15 @@ private:
 
     bool isResetTurnedIntoPowerOff(void);
 
+    /** @name Disk encryption support
+     * @{ */
+    HRESULT consoleParseDiskEncryption(const char *psz, const char **ppszEnd);
+    HRESULT configureEncryptionForDisk(const char *pszUuid);
+    HRESULT clearDiskEncryptionKeysOnAllAttachments(void);
+    int consoleParseKeyValue(const char *psz, const char **ppszEnd,
+                             char **ppszKey, char **ppszVal);
+    /** @} */
+
     /** @name Teleporter support
      * @{ */
     static DECLCALLBACK(int)    teleporterSrcThreadWrapper(RTTHREAD hThread, void *pvUser);
@@ -841,6 +922,15 @@ private:
 
     MediumAttachmentMap mapMediumAttachments;
 
+    /** Map of secret keys used for disk encryption. */
+    SecretKeyMap         m_mapSecretKeys;
+
+    /** Pointer to the key consumer -> provider (that's us) callbacks. */
+    struct MYPDMISECKEY : public PDMISECKEY
+    {
+        Console *pConsole;
+    } *mpIfSecKey;
+
 /* Note: FreeBSD needs this whether netflt is used or not. */
 #if ((defined(RT_OS_LINUX) && !defined(VBOX_WITH_NETFLT)) || defined(RT_OS_FREEBSD))
     Utf8Str      maTAPDeviceName[8];
