@@ -756,6 +756,7 @@ public:
 
     void getLogFolder(Utf8Str &aLogFolder);
     Utf8Str queryLogFilename(ULONG idx);
+    Utf8Str i_getStartupLogFilename(void);
 
     void composeSavedStateFilename(Utf8Str &strStateFilePath);
 
@@ -1212,7 +1213,7 @@ private:
     HRESULT setMachineState(MachineState_T aMachineState);
     HRESULT updateMachineStateOnClient();
 
-    HRESULT mRemoveSavedState;
+    bool mRemoveSavedState;
 
     ConsoleTaskData mConsoleTaskData;
 
