@@ -49,23 +49,26 @@
 *******************************************************************************/
 typedef struct RTHTTPINTERNAL
 {
-    /** magic value */
-    uint32_t u32Magic;
-    /** cURL handle */
-    CURL *pCurl;
-    long lLastResp;
+    /** Magic value. */
+    uint32_t            u32Magic;
+    /** cURL handle. */
+    CURL                *pCurl;
+    /** The last response code. */
+    long                lLastResp;
     /** custom headers */
-    struct curl_slist *pHeaders;
-    /** CA certificate for HTTPS authentication check */
-    char *pcszCAFile;
-    /** abort the current HTTP request if true */
-    bool fAbort;
+    struct curl_slist   *pHeaders;
+    /** CA certificate for HTTPS authentication check. */
+    char                *pcszCAFile;
+    /** Abort the current HTTP request if true. */
+    bool                fAbort;
+    /** The location field for 301 responses. */
+    char                *pszRedirLocation;
 } RTHTTPINTERNAL;
 typedef RTHTTPINTERNAL *PRTHTTPINTERNAL;
 
 typedef struct RTHTTPMEMCHUNK
 {
-    char *pszMem;
+    uint8_t *pu8Mem;
     size_t cb;
 } RTHTTPMEMCHUNK;
 typedef RTHTTPMEMCHUNK *PRTHTTPMEMCHUNK;
@@ -135,6 +138,9 @@ RTR3DECL(void) RTHttpDestroy(RTHTTP hHttp)
     if (pHttpInt->pcszCAFile)
         RTStrFree(pHttpInt->pcszCAFile);
 
+    if (pHttpInt->pszRedirLocation)
+        RTStrFree(pHttpInt->pszRedirLocation);
+
     RTMemFree(pHttpInt);
 
     curl_global_cleanup();
@@ -145,12 +151,12 @@ static DECLCALLBACK(size_t) rtHttpWriteData(void *pvBuf, size_t cb, size_t n, vo
     PRTHTTPMEMCHUNK pMem = (PRTHTTPMEMCHUNK)pvUser;
     size_t cbAll = cb * n;
 
-    pMem->pszMem = (char*)RTMemRealloc(pMem->pszMem, pMem->cb + cbAll + 1);
-    if (pMem->pszMem)
+    pMem->pu8Mem = (uint8_t*)RTMemRealloc(pMem->pu8Mem, pMem->cb + cbAll + 1);
+    if (pMem->pu8Mem)
     {
-        memcpy(&pMem->pszMem[pMem->cb], pvBuf, cbAll);
+        memcpy(&pMem->pu8Mem[pMem->cb], pvBuf, cbAll);
         pMem->cb += cbAll;
-        pMem->pszMem[pMem->cb] = '\0';
+        pMem->pu8Mem[pMem->cb] = '\0';
     }
     return cbAll;
 }
@@ -174,6 +180,18 @@ RTR3DECL(int) RTHttpAbort(RTHTTP hHttp)
     return VINF_SUCCESS;
 }
 
+RTR3DECL(int) RTHttpGetRedirLocation(RTHTTP hHttp, char **ppszRedirLocation)
+{
+    PRTHTTPINTERNAL pHttpInt = hHttp;
+    RTHTTP_VALID_RETURN(pHttpInt);
+
+    if (!pHttpInt->pszRedirLocation)
+        return VERR_HTTP_NOT_FOUND;
+
+    *ppszRedirLocation = RTStrDup(pHttpInt->pszRedirLocation);
+    return VINF_SUCCESS;
+}
+
 RTR3DECL(int) RTHttpUseSystemProxySettings(RTHTTP hHttp)
 {
     PRTHTTPINTERNAL pHttpInt = hHttp;
@@ -304,6 +322,9 @@ RTR3DECL(int) RTHttpCertDigest(RTHTTP hHttp, char *pcszCert, size_t cbCert,
                     }
                     else
                         rc = VERR_HTTP_CACERT_WRONG_FORMAT;
+
+                    if (RT_FAILURE(rc))
+                        RTMemFree(*pabSha1);
                 }
                 else
                     rc = VERR_NO_MEMORY;
@@ -319,12 +340,6 @@ RTR3DECL(int) RTHttpCertDigest(RTHTTP hHttp, char *pcszCert, size_t cbCert,
     else
         rc = VERR_INTERNAL_ERROR;
 
-    if (RT_FAILURE(rc))
-    {
-        RTMemFree(*pabSha512);
-        RTMemFree(*pabSha1);
-    }
-
     return rc;
 }
 
@@ -353,6 +368,12 @@ RTR3DECL(int) RTHttpSetCAFile(RTHTTP hHttp, const char *pcszCAFile)
 static int rtHttpGetCalcStatus(PRTHTTPINTERNAL pHttpInt, int rcCurl)
 {
     int rc = VERR_INTERNAL_ERROR;
+
+    if (pHttpInt->pszRedirLocation)
+    {
+        RTStrFree(pHttpInt->pszRedirLocation);
+        pHttpInt->pszRedirLocation = NULL;
+    }
     if (rcCurl == CURLE_OK)
     {
         curl_easy_getinfo(pHttpInt->pCurl, CURLINFO_RESPONSE_CODE, &pHttpInt->lLastResp);
@@ -364,6 +385,16 @@ static int rtHttpGetCalcStatus(PRTHTTPINTERNAL pHttpInt, int rcCurl)
                 /* empty response */
                 rc = VINF_SUCCESS;
                 break;
+            case 301:
+            {
+                const char *pszRedirect;
+                curl_easy_getinfo(pHttpInt->pCurl, CURLINFO_REDIRECT_URL, &pszRedirect);
+                size_t cb = strlen(pszRedirect);
+                if (cb > 0 && cb < 2048)
+                    pHttpInt->pszRedirLocation = RTStrDup(pszRedirect);
+                rc = VERR_HTTP_REDIRECTED;
+                break;
+            }
             case 400:
                 /* bad request */
                 rc = VERR_HTTP_BAD_REQUEST;
@@ -413,7 +444,7 @@ static int rtHttpGetCalcStatus(PRTHTTPINTERNAL pHttpInt, int rcCurl)
     return rc;
 }
 
-RTR3DECL(int) RTHttpGet(RTHTTP hHttp, const char *pcszUrl, char **ppszResponse)
+RTR3DECL(int) rtHttpGet(RTHTTP hHttp, const char *pcszUrl, uint8_t **ppvResponse, size_t *pcb)
 {
     PRTHTTPINTERNAL pHttpInt = hHttp;
     RTHTTP_VALID_RETURN(pHttpInt);
@@ -459,12 +490,29 @@ RTR3DECL(int) RTHttpGet(RTHTTP hHttp, const char *pcszUrl, char **ppszResponse)
 
     rcCurl = curl_easy_perform(pHttpInt->pCurl);
     int rc = rtHttpGetCalcStatus(pHttpInt, rcCurl);
-    *ppszResponse = chunk.pszMem;
+    *ppvResponse = chunk.pu8Mem;
+    *pcb = chunk.cb;
 
     return rc;
 }
 
 
+RTR3DECL(int) RTHttpGetText(RTHTTP hHttp, const char *pcszUrl, char **ppszResponse)
+{
+    uint8_t *pv;
+    size_t cb;
+    int rc = rtHttpGet(hHttp, pcszUrl, &pv, &cb);
+    *ppszResponse = (char*)pv;
+    return rc;
+}
+
+
+RTR3DECL(int) RTHttpGetBinary(RTHTTP hHttp, const char *pcszUrl, void **ppvResponse, size_t *pcb)
+{
+    return rtHttpGet(hHttp, pcszUrl, (uint8_t**)ppvResponse, pcb);
+}
+
+
 static size_t rtHttpWriteDataToFile(void *pvBuf, size_t cb, size_t n, void *pvUser)
 {
     size_t cbAll = cb * n;
