@@ -216,6 +216,7 @@ static int rtSystemWinQueryOSVersion(RTSYSOSINFO enmInfo, char *pszInfo, size_t
                 case kRTWinOSType_7:            strcpy(szTmp, "Windows 7"); break;
                 case kRTWinOSType_8:            strcpy(szTmp, "Windows 8"); break;
                 case kRTWinOSType_81:           strcpy(szTmp, "Windows 8.1"); break;
+                case kRTWinOSType_10:           strcpy(szTmp, "Windows 10"); break;
 
                 case kRTWinOSType_NT_UNKNOWN:
                     RTStrPrintf(szTmp, sizeof(szTmp), "Unknown NT v%u.%u",
@@ -260,8 +261,13 @@ static int rtSystemWinQueryOSVersion(RTSYSOSINFO enmInfo, char *pszInfo, size_t
             else if (g_WinOsInfoEx.szCSDVersion[0])
             {
                 /* just copy the entire string. */
-                memcpy(szTmp, g_WinOsInfoEx.szCSDVersion, sizeof(g_WinOsInfoEx.szCSDVersion));
-                szTmp[sizeof(g_WinOsInfoEx.szCSDVersion)] = '\0';
+                char *pszTmp = szTmp;
+                int rc = RTUtf16ToUtf8Ex(g_WinOsInfoEx.szCSDVersion, RT_ELEMENTS(g_WinOsInfoEx.szCSDVersion),
+                                         &pszTmp, sizeof(szTmp), NULL);
+                if (RT_SUCCESS(rc))
+                    RTStrStripR(szTmp);
+                else
+                    szTmp[0] = '\0';
                 AssertCompile(sizeof(szTmp) > sizeof(g_WinOsInfoEx.szCSDVersion));
             }
             else
