@@ -158,18 +158,19 @@ RT_C_DECLS_BEGIN
 #define HM_CHANGED_GUEST_SYSENTER_CS_MSR         RT_BIT(13)
 #define HM_CHANGED_GUEST_SYSENTER_EIP_MSR        RT_BIT(14)
 #define HM_CHANGED_GUEST_SYSENTER_ESP_MSR        RT_BIT(15)
+#define HM_CHANGED_GUEST_EFER_MSR                RT_BIT(16)
 /* VT-x specific state. */
-#define HM_CHANGED_VMX_GUEST_AUTO_MSRS           RT_BIT(16)
-#define HM_CHANGED_VMX_GUEST_ACTIVITY_STATE      RT_BIT(17)
-#define HM_CHANGED_VMX_GUEST_APIC_STATE          RT_BIT(18)
-#define HM_CHANGED_VMX_ENTRY_CTLS                RT_BIT(19)
-#define HM_CHANGED_VMX_EXIT_CTLS                 RT_BIT(20)
+#define HM_CHANGED_VMX_GUEST_AUTO_MSRS           RT_BIT(17)
+#define HM_CHANGED_VMX_GUEST_ACTIVITY_STATE      RT_BIT(18)
+#define HM_CHANGED_VMX_GUEST_APIC_STATE          RT_BIT(19)
+#define HM_CHANGED_VMX_ENTRY_CTLS                RT_BIT(20)
+#define HM_CHANGED_VMX_EXIT_CTLS                 RT_BIT(21)
 /* AMD-V specific state. */
-#define HM_CHANGED_SVM_GUEST_EFER_MSR            RT_BIT(16)
 #define HM_CHANGED_SVM_GUEST_APIC_STATE          RT_BIT(17)
 #define HM_CHANGED_SVM_RESERVED1                 RT_BIT(18)
 #define HM_CHANGED_SVM_RESERVED2                 RT_BIT(19)
 #define HM_CHANGED_SVM_RESERVED3                 RT_BIT(20)
+#define HM_CHANGED_SVM_RESERVED4                 RT_BIT(21)
 
 #define HM_CHANGED_ALL_GUEST                     (  HM_CHANGED_GUEST_CR0                \
                                                   | HM_CHANGED_GUEST_CR3                \
@@ -187,13 +188,14 @@ RT_C_DECLS_BEGIN
                                                   | HM_CHANGED_GUEST_SYSENTER_CS_MSR    \
                                                   | HM_CHANGED_GUEST_SYSENTER_EIP_MSR   \
                                                   | HM_CHANGED_GUEST_SYSENTER_ESP_MSR   \
+                                                  | HM_CHANGED_GUEST_EFER_MSR           \
                                                   | HM_CHANGED_VMX_GUEST_AUTO_MSRS      \
                                                   | HM_CHANGED_VMX_GUEST_ACTIVITY_STATE \
                                                   | HM_CHANGED_VMX_GUEST_APIC_STATE     \
                                                   | HM_CHANGED_VMX_ENTRY_CTLS           \
                                                   | HM_CHANGED_VMX_EXIT_CTLS)
 
-#define HM_CHANGED_HOST_CONTEXT                  RT_BIT(21)
+#define HM_CHANGED_HOST_CONTEXT                  RT_BIT(22)
 
 /* Bits shared between host and guest. */
 #define HM_CHANGED_HOST_GUEST_SHARED_STATE       (  HM_CHANGED_GUEST_CR0                \
@@ -342,7 +344,10 @@ typedef struct HM
 
     /** Set when TPR patching is active. */
     bool                        fTPRPatchingActive;
-    bool                        u8Alignment[7];
+    bool                        u8Alignment[3];
+
+    /** Host kernel flags that HM might need to know (SUPKERNFEATURES_XXX). */
+    uint32_t                    uHostKernelFeatures;
 
     /** Maximum ASID allowed. */
     uint32_t                    uMaxAsid;
@@ -421,6 +426,9 @@ typedef struct HM
 
         /** Host EFER value (set by ring-0 VMX init) */
         uint64_t                    u64HostEfer;
+        /** Whether the CPU supports VMCS fields for swapping EFER. */
+        bool                        fSupportsVmcsEfer;
+        bool                        afAlignment1[7];
 
         /** VMX MSR values */
         VMXMSRS                     Msrs;
@@ -667,9 +675,12 @@ typedef struct HMCPU
         /** Virtual address of the VM-exit MSR-load area (used for host MSRs). */
         R0PTRTYPE(void *)           pvHostMsr;
 
-        /** Number of automatically loaded/restored guest MSRs during the world switch. */
+        /** Number of automatically loaded/restored guest MSRs during
+         *  the world switch. */
         uint32_t                    cGuestMsrs;
-        uint32_t                    uAlignment;
+        /** Number of automatically loaded/restored host MSRs during
+         *  the world switch. */
+        uint32_t                    cHostMsrs;
 #endif /* VBOX_WITH_AUTO_MSR_LOAD_RESTORE */
 
         /** The cached APIC-base MSR used for identifying when to map the HC physical APIC-access page. */
